import math
from enum import Enum
from typing import Literal, Union

from pydantic import BaseModel

from pycivil.EXAUtils.EXAExceptions import EXAExceptions
from pycivil.EXAStructural.lawcodes.codeEC212 import Aggregates, Moisture, SteelShapingType
from pycivil.EXAStructural.codes import Code, CodeEnum


class Steel_NTC2008_Enum(str, Enum):
    B450A = "B450A"
    B450C = "B450C"


class Steel_NTC2018_Enum(str, Enum):
    B450A = "B450A"
    B450C = "B450C"


class Concrete_NTC2018_Enum(str, Enum):
    C12_15 = "C12/15"
    C16_20 = "C16/20"
    C20_25 = "C20/25"
    C25_30 = "C25/30"
    C28_35 = "C28/35"
    C30_37 = "C30/37"
    C35_45 = "C35/45"
    C40_50 = "C40/50"
    C45_55 = "C45/55"
    C50_60 = "C50/60"
    C55_67 = "C55/67"
    C60_75 = "C60/75"
    C70_85 = "C70/85"
    C80_95 = "C80/95"
    C90_105 = "C90/105"


class Concrete_NTC2008_Enum(str, Enum):
    C12_15 = "C12/15"
    C16_20 = "C16/20"
    C20_25 = "C20/25"
    C25_30 = "C25/30"
    C28_35 = "C28/35"
    C32_40 = "C32/40"
    C35_45 = "C35/45"
    C40_50 = "C40/50"
    C45_55 = "C45/55"
    C50_60 = "C50/60"
    C55_67 = "C55/67"
    C60_75 = "C60/75"
    C70_85 = "C70/85"
    C80_95 = "C80/95"
    C90_105 = "C90/105"


class Environment_Concrete_Enum(str, Enum):
    AGGRESSIVITY_NOT = "not aggressive"
    AGGRESSIVITY_LOW = "low aggressive"
    AGGRESSIVITY_HIGHT = "hight aggressive"
    AGGRESSIVITY_ND = "undefined"


class Environment_Steel_Enum(str, Enum):
    SENSITIVITY_LOW = "not sensitive"
    SENSITIVITY_HIGHT = "sensitive"
    SENSITIVITY_ND = "undefined"


class Material:
    def __init__(self, ids=-1, descr="generic"):
        self.__ids = ids
        self.__descr = descr
        self.__byCode = ""
        self.__catStr = ""

        # Need to know value derived from manual or by code
        self.__setByCode = False

    def setId(self, ids: int) -> None:
        self.__ids = ids

    def getId(self) -> int:
        return self.__ids

    def unsetByCode(self):
        self.__setByCode = False

    def setByCode(self, codeObj: Code, catstr: str = "") -> None:
        if isinstance(codeObj, Code):
            self.__byCode = codeObj.codeStr()
            self.__catStr = catstr
            self.__setByCode = True
        else:
            raise Exception("Only one [Code] argument !!!")

    def isSetByCode(self) -> bool:
        return self.__setByCode

    def codeStr(self) -> str:
        return self.__byCode

    def catStr(self) -> str:
        return self.__catStr

    def getMatDescr(self):
        return self.__descr

    def setMatDescr(self, descr: str) -> None:
        self.__descr = descr

    def __str__(self):
        dispstr = "Material Object: \n"
        dispstr = dispstr + "--------------- \n"
        dispstr = dispstr + "  id: " + str(self.__ids) + "\n"
        dispstr = dispstr + "desc: " + str(self.__descr) + "\n"
        dispstr = dispstr + "         code: " + str(self.__byCode) + "\n"
        dispstr = dispstr + "setted byCode: " + str(self.__setByCode) + "\n"
        return dispstr


class Concrete(Material):

    tab_fck = {
        "EC2": {
            "C12/15": 12,
            "C16/20": 16,
            "C20/25": 20,
            "C25/30": 25,
            "C30/37": 30,
            "C32/40": 32,
            "C35/45": 35,
            "C40/50": 40,
            "C45/55": 45,
            "C50/60": 50,
            "C55/67": 55,
            "C60/75": 60,
            "C70/85": 70,
            "C80/95": 80,
            "C90/105": 90,
        },
        "EC2:ITA": {
            "C12/15": 12,
            "C16/20": 16,
            "C20/25": 20,
            "C25/30": 25,
            "C30/37": 30,
            "C32/40": 32,
            "C35/45": 35,
            "C40/50": 40,
            "C45/55": 45,
            "C50/60": 50,
            "C55/67": 55,
            "C60/75": 60,
            "C70/85": 70,
            "C80/95": 80,
            "C90/105": 90,
        },
        "NTC2018": {
            "C12/15": 12,
            "C16/20": 16,
            "C20/25": 20,
            "C25/30": 25,
            "C30/37": 30,
            "C35/45": 35,
            "C40/50": 40,
            "C45/55": 45,
            "C50/60": 50,
            "C55/67": 55,
            "C60/75": 60,
            "C70/85": 70,
            "C80/95": 80,
            "C90/105": 90,
        },
        "NTC2018:RFI": {
            "C12/15": 12,
            "C16/20": 16,
            "C20/25": 20,
            "C25/30": 25,
            "C30/37": 30,
            "C35/45": 35,
            "C40/50": 40,
            "C45/55": 45,
            "C50/60": 50,
            "C55/67": 55,
            "C60/75": 60,
            "C70/85": 70,
            "C80/95": 80,
            "C90/105": 90,
        },
        "NTC2008": {
            "C8/10": 8,
            "C12/15": 12,
            "C16/20": 16,
            "C20/25": 20,
            "C25/30": 25,
            "C28/35": 28,
            "C32/40": 32,
            "C35/45": 35,
            "C40/50": 40,
            "C45/55": 45,
            "C50/60": 50,
            "C55/67": 55,
            "C60/75": 60,
            "C70/85": 70,
            "C80/95": 80,
            "C90/105": 90,
        },
    }

    tab_Rck = {
        "EC2": {
            "C12/15": 15,
            "C16/20": 20,
            "C20/25": 25,
            "C25/30": 30,
            "C30/37": 37,
            "C32/40": 40,
            "C35/45": 45,
            "C40/50": 50,
            "C45/55": 55,
            "C50/60": 60,
            "C55/67": 67,
            "C60/75": 75,
            "C70/85": 85,
            "C80/95": 95,
            "C90/105": 105,
        },
        "EC2:ITA": {
            "C12/15": 15,
            "C16/20": 20,
            "C20/25": 25,
            "C25/30": 30,
            "C30/37": 37,
            "C32/40": 40,
            "C35/45": 45,
            "C40/50": 50,
            "C45/55": 55,
            "C50/60": 60,
            "C55/67": 67,
            "C60/75": 75,
            "C70/85": 85,
            "C80/95": 95,
            "C90/105": 105,
        },
        "NTC2018": {
            "C12/15": 15,
            "C16/20": 20,
            "C20/25": 25,
            "C25/30": 30,
            "C30/37": 37,
            "C35/45": 45,
            "C40/50": 50,
            "C45/55": 55,
            "C50/60": 60,
            "C55/67": 67,
            "C60/75": 75,
            "C70/85": 85,
            "C80/95": 95,
            "C90/105": 105,
        },
        "NTC2018:RFI": {
            "C12/15": 15,
            "C16/20": 20,
            "C20/25": 25,
            "C25/30": 30,
            "C30/37": 37,
            "C35/45": 45,
            "C40/50": 50,
            "C45/55": 55,
            "C50/60": 60,
            "C55/67": 67,
            "C60/75": 75,
            "C70/85": 85,
            "C80/95": 95,
            "C90/105": 105,
        },
        "NTC2008": {
            "C8/10": 10,
            "C12/15": 15,
            "C16/20": 20,
            "C20/25": 25,
            "C25/30": 30,
            "C28/35": 35,
            "C32/40": 40,
            "C35/45": 45,
            "C40/50": 50,
            "C45/55": 55,
            "C50/60": 60,
            "C55/67": 67,
            "C60/75": 75,
            "C70/85": 85,
            "C80/95": 95,
            "C90/105": 105,
        },
    }

    def __init__(self, ids: int = -1, descr: str = "concrete") -> None:

        self.__Rck: float = 0.0
        self.__fck: float = 0.0
        self.__fctm: float = 0.0
        self.__fcm: float = 0.0
        self.__ec2: float = 0.0
        self.__ecu: float = 0.0
        self.__Ecm: float = 0.0
        self.__fct_crack: float = 0.0

        # EC2:
        # resistenza effettiva della zona di compressione. Infatti
        # eta*fcd è l'altezza dello stress block
        #
        # NTC2018:
        # coefficiente riduttivo di lunga durata 0.85
        self.__eta: float = 0.0

        # EC2:
        # profondità della zona compressa del calcestruzzo nello stress-block
        #
        # NTC2018:
        # come per EC2
        self.__lambda: float = 0.0

        # EC2:
        # the coefficient taking account of long term effects on the compressive
        # strength and of unfavourable effects resulting from the way the load
        # is applied.
        #
        # NTC2018:
        # coefficiente riduttivo di lunga durata 0.85
        self.__alphacc: float = 0.0
        self.__alphacc_fire: float = 0.0

        self.__environnment = "not aggressive"

        self.__gammac: float = 0.0
        self.__gammac_fire: float = 0.0
        # EC2: limite calceestruzzo combo RARA
        self.__k1: float = 0.0
        # EC2: limite calceestruzzo combo QP
        self.__k2: float = 0.0
        self.__sigmac_max_c: float = 0.0
        self.__sigmac_max_q: float = 0.0

        # EC212:
        # per il fire design
        self.__aggregates = Aggregates.CALCAREOUS
        self.__moisture = Moisture.v00

        Material.__init__(self, ids, descr)

    def setByCode(self, codeObj: Code, catstr: str = "") -> None:

        if codeObj.codeStr() in self.tab_fck:
            if catstr in self.tab_fck[codeObj.codeStr()]:

                Material.setByCode(self, codeObj, catstr)
                self.__fck = self.tab_fck[codeObj.codeStr()][catstr]
                self.__Rck = self.tab_Rck[codeObj.codeStr()][catstr]

                if (
                    codeObj.codeStr() == "EC2"
                    or codeObj.codeStr() == "EC2:ITA"
                    or codeObj.codeStr() == "NTC2008"
                    or codeObj.codeStr() == "NTC2018"
                    or codeObj.codeStr() == "NTC2018:RFI"
                ):
                    if codeObj.codeStr() == "EC2":
                        self.__alphacc = 1.00
                        self.__alphacc_fire = 1.00
                    else:
                        self.__alphacc = 0.85
                        self.__alphacc_fire = 1.00

                    self.__fcm = self.__fck + 8

                    self.__gammac = 1.5
                    self.__gammac_fire = 1.0

                    if codeObj.codeStr() == "NTC2018:RFI":
                        self.__k1 = 0.55
                        self.__k2 = 0.40
                    else:
                        self.__k1 = 0.6
                        self.__k2 = 0.45

                    self.__sigmac_max_c = self.__k1 * self.__fck
                    self.__sigmac_max_q = self.__k2 * self.__fck

                    # Mega pascal from Giga pascal * 1000
                    self.__Ecm = 22 * pow((self.__fcm / 10), 0.3) * 1000

                    fck_min = 12.0
                    if codeObj.codeStr() == "NTC2008":
                        fck_min = 8.0

                    if fck_min <= self.__fck < 50.0:
                        self.__fctm = 0.3 * (self.__fck ** (2 / 3))
                        self.__ec2 = 0.002
                        self.__ecu = 0.0035
                        self.__lambda = 0.8
                        self.__eta = 1.0

                    elif 50.0 <= self.__fck <= 90.0:
                        self.__fctm = 2.12 * math.log(1 + self.__fcm / 10)
                        self.__ec2 = (2.0 + 0.085 * (self.__fck - 50.0) ** 0.53) / 1000
                        self.__ecu = (
                            2.6 + 35 * (((90.0 - self.__fck) / 100) ** 4)
                        ) / 1000
                        self.__lambda = 0.8 - (self.__fck - 50.0) / 400.0
                        self.__eta = 1.0 + (self.__fck - 50.0) / 200.0

                    else:
                        raise EXAExceptions(
                            "(EXAStructural)-0004",
                            f"fck must be >={fck_min:.3f} and <=90.0",
                            self.__fck,
                        )

                    # Paragraph 4.1.2.2.4 formula [4.1.13] in NTC2018
                    #
                    self.__fct_crack = self.__fctm / 1.2

                else:
                    raise EXAExceptions(
                        "(EXAStructural)-0005", "not implemented", codeObj.codeStr()
                    )
            else:
                raise EXAExceptions("(EXAStructural)-0003", "fck class unknown", catstr)
        else:
            raise EXAExceptions(
                "(EXAStructural)-0002", "codeStr unknown", codeObj.codeStr()
            )

    def set_fck(self, fck):
        self.__fck = fck
        super().unsetByCode()

    def set_Rck(self, Rck):
        self.__Rck = Rck
        super().unsetByCode()

    def set_fctm(self, fctm):
        self.__fctm = fctm
        super().unsetByCode()

    def set_fcm(self, fcm):
        self.__fcm = fcm
        super().unsetByCode()

    def set_ec2(self, ec2):
        self.__ec2 = ec2
        super().unsetByCode()

    def set_ecu(self, ecu):
        self.__ecu = ecu
        super().unsetByCode()

    def set_k1(self, k1):
        self.__k1 = k1
        super().unsetByCode()

    def set_k2(self, k2):
        self.__k2 = k2
        super().unsetByCode()

    def get_k1(self):
        return self.__k1

    def get_k2(self):
        return self.__k2

    def set_lambda(self, lm):
        self.__lambda = lm
        super().unsetByCode()

    def get_fct_crack(self):
        return self.__fct_crack

    def set_fct_crack(self, fct):
        self.__fct_crack = fct
        super().unsetByCode()

    def set_eta(self, eta: float) -> None:
        self.__eta = eta
        super().unsetByCode()

    def set_alphacc(self, alphacc: float) -> None:
        self.__alphacc = alphacc
        super().unsetByCode()

    def set_alphacc_fire(self, alphacc: float) -> None:
        self.__alphacc_fire = alphacc
        super().unsetByCode()

    def set_gammac(self, gammac: float) -> None:
        self.__gammac = gammac
        Material.unsetByCode(self)

    def set_gammac_fire(self, gammac_fire):
        self.__gammac_fire = gammac_fire
        Material.unsetByCode(self)

    def get_gammac_fire(self):
        return self.__gammac_fire

    def set_sigmac_max_c(self, sigmac_max_c):
        self.__sigmac_max_c = sigmac_max_c
        Material.unsetByCode(self)

    def set_sigmac_max_q(self, sigmac_max_q):
        self.__sigmac_max_q = sigmac_max_q
        Material.unsetByCode(self)

    def get_fck(self) -> float:
        return self.__fck

    def get_Rck(self) -> float:
        return self.__Rck

    def get_fctm(self) -> float:
        return self.__fctm

    def get_fcm(self) -> float:
        return self.__fcm

    def get_Ecm(self) -> float:
        return self.__Ecm

    def get_ec2(self) -> float:
        return self.__ec2

    def get_ecu(self) -> float:
        return self.__ecu

    def get_lambda(self) -> float:
        return self.__lambda

    def get_eta(self) -> float:
        return self.__eta

    def get_alphacc(self) -> float:
        return self.__alphacc

    def get_alphacc_fire(self) -> float:
        return self.__alphacc_fire

    def getEnvironment(self):
        return self.__environnment

    def getEnvironmentTr(self, lan: str="IT") -> str:
        if lan == "IT":
            if self.__environnment == "hight-aggressive":
                return "fortemente aggressivo"
            if self.__environnment == "low-aggressive":
                return "aggressivo"
            if self.__environnment == "not aggressive":
                return "non aggressivo"
        return ""

    def cal_fcd(self) -> float:
        return self.__alphacc * self.__fck / self.__gammac

    def isEnvironmentHightAggressive(self):
        return self.__environnment == "hight-aggressive"

    def isEnvironmentAggressive(self):
        return self.__environnment == "low-aggressive"

    def isEnvironmentNotAggressive(self):
        return self.__environnment == "not aggressive"

    def setEnvironmentHightAggressive(self):
        self.__environnment = "hight-aggressive"

    def setEnvironmentAggressive(self):
        self.__environnment = "low-aggressive"

    def setEnvironmentNotAggressive(self):
        self.__environnment = "not aggressive"

    def setEnvironment(
        self, tp: Literal["hight-aggressive", "low-aggressive", "not aggressive"]
    ) -> None:
        self.__environnment = tp

    def get_gammac(self) -> float:
        return self.__gammac

    def get_sigmac_max_c(self) -> float:
        return self.__sigmac_max_c

    def cal_sigmac_max_c(self) -> float:
        return self.__k1 * self.__fck

    def cal_sigmac_max_q(self) -> float:
        return self.__k2 * self.__fck

    def get_sigmac_max_q(self) -> float:
        return self.__sigmac_max_q

    def setAggregates(self, aggregates: Aggregates) -> None:
        self.__aggregates = aggregates

    def getAggregates(self) -> Aggregates:
        return self.__aggregates

    def setMoisture(self, moisture: Moisture) -> None:
        self.__moisture = moisture

    def getMoisture(self) -> Moisture:
        return self.__moisture

    def __str__(self):
        dispstr = super().__str__()
        dispstr = dispstr + "Data embedded-->\n"
        dispstr = dispstr + "        Code = " + str(self.codeStr()) + "\n"
        dispstr = dispstr + "       catStr = " + str(self.catStr()) + "\n"
        dispstr = dispstr + "         fck = " + str(self.__fck) + "\n"
        dispstr = dispstr + "        fctm = " + str(self.__fctm) + "\n"
        dispstr = dispstr + "         fcm = " + str(self.__fcm) + "\n"
        dispstr = dispstr + "         Ecm = " + str(self.__Ecm) + "\n"
        dispstr = dispstr + "         ec2 = " + str(self.__ec2) + "\n"
        dispstr = dispstr + "         ecu = " + str(self.__ecu) + "\n"
        dispstr = dispstr + "         Ecm = " + str(self.__ecu) + "\n"
        dispstr = dispstr + "      lambda = " + str(self.__lambda) + "\n"
        dispstr = dispstr + "         eta = " + str(self.__eta) + "\n"
        dispstr = dispstr + "     alphacc = " + str(self.__alphacc) + "\n"
        dispstr = dispstr + "alphacc fire = " + str(self.__alphacc_fire) + "\n"
        dispstr = dispstr + "      gammac = " + str(self.__gammac) + "\n"
        dispstr = dispstr + " gammac_fire = " + str(self.__gammac_fire) + "\n"
        dispstr = dispstr + "sigmac max c = " + str(self.__sigmac_max_c) + "\n"
        dispstr = dispstr + "sigmac max q = " + str(self.__sigmac_max_q) + "\n"
        dispstr = dispstr + "environnment = " + str(self.__environnment) + "\n"
        return dispstr.replace("\n", "\n  | ")


class ConcreteModel(BaseModel):
    code: CodeEnum = CodeEnum.NTC2018
    mat: Union[
        Concrete_NTC2008_Enum, Concrete_NTC2018_Enum
    ] = Concrete_NTC2008_Enum.C20_25
    descr: str = ""
    bycode: bool = True
    fck: float = -1
    fctm: float = -1
    fcm: float = -1
    ec2: float = -1
    ecu: float = -1
    Ecm: float = -1
    eta: float = -1
    llambda: float = -1
    alphacc: float = -1
    alphacc_fire: float = -1
    gammac: float = -1
    sigmac_max_c: float = -1
    sigmac_max_q: float = -1
    environment: Environment_Concrete_Enum = Environment_Concrete_Enum.AGGRESSIVITY_NOT
    aggregates: Aggregates = Aggregates.CALCAREOUS
    moisture: Moisture = Moisture.v00

    def fromMaterial(self, mat: Material) -> bool:
        if not isinstance(mat, Concrete):
            return False

        if mat.codeStr() != '':
            self.code = CodeEnum(mat.codeStr())

        if self.code == CodeEnum.NTC2018:
            if mat.catStr() != '':
                self.mat = Concrete_NTC2018_Enum(mat.catStr())
        elif self.code == CodeEnum.NTC2008:
            if mat.catStr() != '':
                self.mat = Concrete_NTC2008_Enum(mat.catStr())
        else:
            return False

        self.descr = mat.getMatDescr()
        self.bycode = mat.isSetByCode()
        self.fck = mat.get_fck()
        self.fctm = mat.get_fctm()
        self.fcm = mat.get_fcm()
        self.ec2 = mat.get_ec2()
        self.ecu = mat.get_ecu()
        self.Ecm = mat.get_Ecm()
        self.eta = mat.get_eta()
        self.llambda = mat.get_lambda()
        self.alphacc = mat.get_alphacc()
        self.alphacc_fire = mat.get_alphacc_fire()
        self.gammac = mat.get_gammac()
        self.sigmac_max_c = mat.get_sigmac_max_c()
        self.sigmac_max_q = mat.get_sigmac_max_q()
        self.environment = Environment_Concrete_Enum(mat.getEnvironment())
        self.aggregates = mat.getAggregates()
        self.moisture = mat.getMoisture()
        return True

    def toMaterial(self) -> Concrete:
        mat = Concrete(descr=self.descr)
        if self.bycode:
            codeConcrete = Code(self.code.value)
            mat.setByCode(codeConcrete, self.mat.value)
        else:
            mat.set_alphacc(self.alphacc)
            mat.set_alphacc_fire(self.alphacc_fire)
            mat.set_ec2(self.ec2)
            mat.set_ecu(self.ecu)
            mat.set_eta(self.eta)
            mat.set_fcm(self.fcm)
            mat.set_fck(self.fck)
            mat.set_fctm(self.fctm)
            mat.set_lambda(self.llambda)
            mat.set_gammac(self.gammac)
            mat.set_sigmac_max_c(self.sigmac_max_c)
            mat.set_sigmac_max_q(self.sigmac_max_q)

        if (
            self.environment
            == self.environment.AGGRESSIVITY_HIGHT
        ):
            mat.setEnvironmentHightAggressive()
        elif (
            self.environment
            == self.environment.AGGRESSIVITY_LOW
        ):
            mat.setEnvironmentAggressive()
        elif (
            self.environment
            == self.environment.AGGRESSIVITY_NOT
        ):
            mat.setEnvironmentNotAggressive()
        else:
            print("ERR: environment unknown !!!")

        mat.setAggregates(self.aggregates)
        mat.setMoisture(self.moisture)
        return mat

class ConcreteSteel(Material):
    tab_steel = {
        "EC2": {
            "B450A": {
                "fyk": 450,
                "fuk": 450 * 1.05,
                "euk": 0.0025,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.05,
                },
            },
            "B450B": {
                "fyk": 450,
                "fuk": 450 * 1.08,
                "euk": 0.0050,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.08,
                },
            },
            "B450C": {
                "fyk": 450,
                "fuk": 540,
                "euk": 0.0075,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {"geq": 1.15, "leq": 1.35},
            },
        },
        "EC2:ITA": {
            "B450A": {
                "fyk": 450,
                "fuk": 450 * 1.05,
                "euk": 0.0025,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.05,
                },
            },
            "B450B": {
                "fyk": 450,
                "fuk": 450 * 1.08,
                "euk": 0.0050,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.08,
                },
            },
            "B450C": {
                "fyk": 450,
                "fuk": 540,
                "euk": 0.0075,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {"geq": 1.15, "leq": 1.35},
            },
        },
        "NTC2008": {
            "B450A": {
                "fyk": 450,
                "fuk": 450 * 1.05,
                "euk": 0.0025,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.05,
                },
            },
            "B450C": {
                "fyk": 450,
                "fuk": 540,
                "euk": 0.0075,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {"geq": 1.15, "leq": 1.35},
            },
        },
        "NTC2018": {
            "B450A": {
                "fyk": 450,
                "fuk": 450 * 1.05,
                "euk": 0.0025,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.05,
                },
            },
            "B450C": {
                "fyk": 450,
                "fuk": 540,
                "euk": 0.0075,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {"geq": 1.15, "leq": 1.35},
            },
        },
        "NTC2018:RFI": {
            "B450A": {
                "fyk": 450,
                "fuk": 450 * 1.05,
                "euk": 0.0025,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.80,
                "(ft/fy)k": {
                    "geq": 1.05,
                },
            },
            "B450C": {
                "fyk": 450,
                "fuk": 540,
                "euk": 0.0075,
                "Es": 210000,
                "gammas": 1.15,
                "gammas_fire": 1.00,
                "sigmas_max_c_ratio": 0.75,
                "(ft/fy)k": {"geq": 1.15, "leq": 1.35},
            },
        },
    }

    def __init__(self, ids: int = -1, descr: str = "concrete steel"):
        self.__fsy: float = 0.0
        self.__Es: float = 0.0
        self.__esy: float = 0.0
        self.__esu: float = 0.0
        self.__fuk: float = 0.0
        self.__gammas: float = 0.0
        self.__gammas_fire: float = 0.0
        self.__shapedType = SteelShapingType.HOT_ROLLED
        self.__sigmas_max_c_ratio: float = 0.0
        self.__sigmas_max_c: float = 0.0
        self.__sensitivity = "not sensitive"

        Material.__init__(self, ids, descr)

    def setByCode(self, codeObj, catstr=""):
        if codeObj.codeStr() in self.tab_steel:
            if catstr in self.tab_steel[codeObj.codeStr()]:

                Material.setByCode(self, codeObj, catstr)

                value = self.tab_steel[codeObj.codeStr()][catstr]["fyk"]
                assert isinstance(value, (int, float))
                self.__fsy = value

                value = self.tab_steel[codeObj.codeStr()][catstr]["fuk"]
                assert isinstance(value, (int, float))
                self.__fuk = value

                value = self.tab_steel[codeObj.codeStr()][catstr]["euk"]
                assert isinstance(value, (int, float))
                self.__esu = value

                value = self.tab_steel[codeObj.codeStr()][catstr]["Es"]
                assert isinstance(value, (int, float))
                self.__Es = value

                value = self.tab_steel[codeObj.codeStr()][catstr]["gammas"]
                assert isinstance(value, (int, float))
                self.__gammas = value

                value = self.tab_steel[codeObj.codeStr()][catstr]["gammas_fire"]
                assert isinstance(value, (int, float))
                self.__gammas_fire = value

                value = self.tab_steel[codeObj.codeStr()][catstr]["sigmas_max_c_ratio"]
                assert isinstance(value, (int, float))
                self.__sigmas_max_c_ratio = value

                self.__sigmas_max_c = self.__sigmas_max_c_ratio * self.__fsy
                self.__esy = self.__fsy / self.__Es
            else:
                raise EXAExceptions("(EXAStructural)-0007", "steel tag unknown", catstr)
        else:
            raise EXAExceptions(
                "(EXAStructural)-0006", "codeStr unknown", codeObj.codeStr()
            )

    def set_fsy(self, fsy):
        super().unsetByCode()
        self.__fsy = fsy

    def set_fuk(self, fuk):
        super().unsetByCode()
        self.__fuk = fuk

    def get_fuk(self):
        return self.__fuk

    def set_Es(self, Es):
        super().unsetByCode()
        self.__Es = Es

    def set_esy(self, esy):
        super().unsetByCode()
        self.__esy = esy

    def set_esu(self, esu):
        super().unsetByCode()
        self.__esu = esu

    def set_gammas(self, gammas):
        super().unsetByCode()
        self.__gammas = gammas

    def set_gammas_fire(self, gammas_fire):
        super().unsetByCode()
        self.__gammas_fire = gammas_fire

    def get_gammas_fire(self) -> float:
        return self.__gammas_fire

    def set_shapingType(self, tp: SteelShapingType) -> None:
        self.__shapedType = tp

    def get_shapingType(self) -> SteelShapingType:
        return self.__shapedType

    def set_sigmas_max_c(self, sigmas_max_c):
        super().unsetByCode()
        self.__sigmas_max_c = sigmas_max_c

    def cal_sigmas_max_c(self):
        return self.__sigmas_max_c_ratio * self.__sigmas_max_c

    def cal_fyd(self):
        return self.__fsy / self.__gammas

    def set_sigmas_max_c_ratio(self, sigmas_max_c_ratio):
        super().unsetByCode()
        self.__sigmas_max_c_ratio = sigmas_max_c_ratio

    def get_sigmas_max_c_ratio(self):
        return self.__sigmas_max_c_ratio

    def get_gammas(self) -> float:
        return self.__gammas

    def get_sigmas_max_c(self) -> float:
        return self.__sigmas_max_c

    def get_fsy(self) -> float:
        return self.__fsy

    def get_Es(self) -> float:
        return self.__Es

    def get_esy(self) -> float:
        return self.__esy

    def get_esu(self) -> float:
        return self.__esu

    def isSteelSensitive(self):
        return self.__sensitivity == "sensitive"

    def setSensitivity(self, tp: Literal["sensitive", "not sensitive"]) -> None:
        self.__sensitivity = tp

    def setEnvironmentSensitive(self):
        self.__sensitivity = "sensitive"

    def setEnvironmentNotSensitive(self):
        self.__sensitivity = "not sensitive"

    def getEnvironment(self):
        return self.__sensitivity

    def getEnvironmentTr(self, lan: str = "IT") -> str:
        if lan == "IT":
            if self.__sensitivity == "sensitive":
                return "sensibile"
            if self.__sensitivity == "not sensitive":
                return "poco sensibile"
        return ""

    def __str__(self):
        dispstr = super().__str__()
        dispstr = dispstr + "Data embedded-->\n"
        dispstr = dispstr + "         fsy = " + str(self.__fsy) + "\n"
        dispstr = dispstr + "          Es = " + str(self.__Es) + "\n"
        dispstr = dispstr + "         esy = " + str(self.__esy) + "\n"
        dispstr = dispstr + "         esu = " + str(self.__esu) + "\n"
        dispstr = dispstr + "      gammas = " + str(self.__gammas) + "\n"
        dispstr = dispstr + " gammas_fire = " + str(self.__gammas_fire) + "\n"
        dispstr = dispstr + "sigmas_max_c = " + str(self.__sigmas_max_c) + "\n"
        dispstr = dispstr + " sensitivity = " + str(self.__sensitivity) + "\n"
        return dispstr.replace("\n", "\n  | ")


class SteelModel(BaseModel):
    code: CodeEnum = CodeEnum.NTC2018
    mat: Union[Steel_NTC2008_Enum, Steel_NTC2018_Enum] = Steel_NTC2018_Enum.B450C
    descr: str = ""
    bycode: bool = True
    fsy: float = -1
    Es: float = -1
    esy: float = -1
    esu: float = -1
    gammas: float = -1
    sigmas_max_c: float = -1
    sensitivity: Environment_Steel_Enum = Environment_Steel_Enum.SENSITIVITY_LOW
    formed: SteelShapingType = SteelShapingType.COLD_ROLLED

    def fromMaterial(self, mat: Material) -> bool:
        if not isinstance(mat, ConcreteSteel):
            return False
        self.code = CodeEnum(mat.codeStr())

        if self.code == CodeEnum.NTC2018:
            self.mat = Steel_NTC2018_Enum(mat.catStr())
        elif self.code == CodeEnum.NTC2008:
            self.mat = Steel_NTC2008_Enum(mat.catStr())
        else:
            return False

        self.descr = mat.getMatDescr()
        self.bycode = mat.isSetByCode()
        self.fsy = mat.get_fsy()
        self.Es = mat.get_Es()
        self.esy = mat.get_esy()
        self.esu = mat.get_esu()
        self.gammas = mat.get_gammas()
        self.sigmas_max_c = mat.get_sigmas_max_c()

        self.sensitivity = Environment_Steel_Enum(mat.getEnvironment())
        self.formed = mat.get_shapingType()

        return True

    def toMaterial(self) -> ConcreteSteel:
        mat = ConcreteSteel(descr=self.descr)

        if self.bycode:
            codeSteel = Code(self.code.value)
            mat.setByCode(codeSteel, self.mat.value)
        else:
            mat.set_Es(self.Es)
            mat.set_esu(self.esu)
            mat.set_esy(self.esy)
            mat.set_fsy(self.fsy)
            mat.set_gammas(self.gammas)
            mat.set_sigmas_max_c(self.sigmas_max_c)

        if (
            self.sensitivity
            == self.sensitivity.SENSITIVITY_HIGHT
        ):
            mat.setEnvironmentSensitive()
        elif (
            self.sensitivity
            == self.sensitivity.SENSITIVITY_LOW
        ):
            mat.setEnvironmentNotSensitive()
        else:
            print("ERR: sensitivity unknown !!!")

        mat.set_shapingType(self.formed)
        return mat


import copy
import math
from enum import Enum
from typing import Any, Dict, List, Literal, Tuple, Union

import numpy as np

import matplotlib
matplotlib.use('Agg')

import matplotlib.pyplot as plt
from matplotlib.axes import Axes
from matplotlib.patches import Circle, PathPatch, Polygon
from matplotlib.path import Path
from matplotlib.ticker import FormatStrFormatter

from pycivil.EXAGeometry.clouds import PointCloud2d
from pycivil.EXAGeometry.geometry import Point2d
from pycivil.EXAStructural.modeler import SectionModeler
from pycivil.EXAStructural.sections import ConcreteSection
from pycivil.EXAUtils.logging import log
from adjustText import adjust_text

class SectionPlotView_Enum(str, Enum):
    SP_VIEW_SECTION = "SP_VIEW_SECTION"
    SP_VIEW_SECTION_NOID = "SP_VIEW_SECTION_NOID"
    SP_VIEW_NODES = "SP_VIEW_NODES"
    SP_VIEW_SOLID_NODES = "SP_VIEW_SOLID_NODES"
    SP_HIDE_SOLID_NODES = "SP_HIDE_SOLID_NODES"
    SP_HIDE_NODES = "SP_HIDE_NODES"
    SP_HIDE_REBAR_ID = "SP_HIDE_REBAR_ID"
    SP_VIEW_GRID = "SP_VIEW_GRID"
    SP_HIDE_GRID = "SP_HIDE_GRID"
    SP_VIEW_TRIANGLES = "SP_VIEW_TRIANGLES"
    SP_VIEW_TRIANGLE_NOID = "SP_VIEW_TRIANGLE_NOID"
    SP_VIEW_MESH = "SP_VIEW_MESH"
    SP_VIEW_STRESSES = "SP_VIEW_STRESSES"


class SectionPlot:
    def __init__(self):
        self.__fig = plt.figure()
        self.__ax = self.__fig.add_subplot(1, 1, 1)
        self.__ax.set_aspect("equal", "datalim")

        # Data to plot
        self.__rcRect: Union[None, ConcreteSection] = None
        self.__modeler: Union[None, SectionModeler] = None
        self.__vertexStresses: Union[Dict[int, float], None] = None
        self.__rebarStresses: Union[Dict[int, float], None] = None

        # Aspect controls
        self.__colorEdge: str = "black"
        self.__textSize: float = 8
        self.__colorTriaLines: str = "blue"
        self.__colorTriaFill: str = "green"
        self.__colorNodes: str = "red"
        self.__plotShapeEdges: bool = True
        self.__plotTriaLines: bool = False
        self.__plotTriaFill: bool = True
        self.__plotTriaShrinkFill: bool = False
        self.__plotTriaShrink: bool = False
        self.__plotNodes: bool = False
        self.__plotNodesId: bool = False
        self.__plotSolidNodes: bool = True
        self.__plotSolidNodesId: bool = True
        self.__plotStresses: bool = False
        self.__plotRebar: bool = True
        self.__plotMeshTria: bool = False
        self.__plotRebarId: bool = True
        self.__plotGrid: bool = False

        self.__ll: Literal[0, 1, 2] = 0

    @property
    def logLevel(self):
        return self.__ll

    @logLevel.setter
    def logLevel(self, value):
        self.__ll = value

    @property
    def plotRebar(self):
        return self.__plotRebar

    @plotRebar.setter
    def plotRebar(self, value):
        self.__plotRebar = value

    @property
    def colorNodes(self):
        return self.__colorNodes

    @colorNodes.setter
    def colorNodes(self, value):
        self.__colorNodes = value

    @property
    def ax(self):
        return self.__ax

    @ax.setter
    def ax(self, value):
        self.__ax = value

    @property
    def colorTriaFill(self):
        return self.__colorTriaFill

    @colorTriaFill.setter
    def colorTriaFill(self, value):
        self.__colorTriaFill = value

    @property
    def plotTriaShrinkFill(self):
        return self.__plotTriaShrinkFill

    @plotTriaShrinkFill.setter
    def plotTriaShrinkFill(self, value):
        self.__plotTriaShrinkFill = value

    @property
    def textSize(self):
        return self.__textSize

    @textSize.setter
    def textSize(self, value):
        self.__textSize = value

    @property
    def vertexStresses(self) -> Union[Dict[int, float], None]:
        return self.__vertexStresses

    @vertexStresses.setter
    def vertexStresses(self, value: Union[Dict[int, float], None] = None):
        self.__vertexStresses = value

    @property
    def rebarStresses(self) -> Union[Dict[int, float], None]:
        return self.__rebarStresses

    @rebarStresses.setter
    def rebarStresses(self, value: Union[Dict[int, float], None] = None):
        self.__rebarStresses = value

    @property
    def plotGrid(self):
        return self.__plotGrid

    @plotGrid.setter
    def plotGrid(self, value):
        self.__plotGrid = value

    @property
    def plotSolidNodesId(self):
        return self.__plotSolidNodesId

    @plotSolidNodesId.setter
    def plotSolidNodesId(self, value):
        self.__plotSolidNodesId = value

    @property
    def modeler(self):
        return self.__modeler

    @modeler.setter
    def modeler(self, value: SectionModeler):
        self.__modeler = value

    def __plotScaled(self, xDataScaled, yDataScaled, refLenght):
        # Scale from reference lenght in data space
        dx = xDataScaled
        dy = yDataScaled
        headScale = 0.02
        i1 = 0
        i2 = 1
        self.__ax.arrow(
            dx[i1],
            dy[i1],
            dx[i2] - dx[i1],
            dy[i2] - dy[i1],
            head_width=refLenght * headScale,
            head_length=refLenght * headScale,
            length_includes_head=True,
        )
        i1 = 1
        i2 = 2
        self.__ax.arrow(
            dx[i1],
            dy[i1],
            dx[i2] - dx[i1],
            dy[i2] - dy[i1],
            head_width=refLenght * headScale,
            head_length=refLenght * headScale,
            length_includes_head=True,
        )
        i1 = 2
        i2 = 0
        self.__ax.arrow(
            dx[i1],
            dy[i1],
            dx[i2] - dx[i1],
            dy[i2] - dy[i1],
            head_width=refLenght * headScale,
            head_length=refLenght * headScale,
            length_includes_head=True,
        )

    @staticmethod
    def __genTriaData(coords: Tuple[float, float, float, float, float, float]) -> Any:
        xData = [coords[0], coords[2], coords[4], coords[0]]
        yData = [coords[1], coords[3], coords[5], coords[1]]

        xg = (xData[0] + xData[1] + xData[2]) / 3
        yg = (yData[0] + yData[1] + yData[2]) / 3

        xDataScaled = [
            (xData[0] - xg) * 0.8 + xg,
            (xData[1] - xg) * 0.8 + xg,
            (xData[2] - xg) * 0.8 + xg,
        ]

        yDataScaled = [
            (yData[0] - yg) * 0.8 + yg,
            (yData[1] - yg) * 0.8 + yg,
            (yData[2] - yg) * 0.8 + yg,
        ]

        polyData = [
            [xData[0], yData[0]],
            [xData[1], yData[1]],
            [xData[2], yData[2]],
        ]

        polyDataScaled = [
            [xDataScaled[0], yDataScaled[0]],
            [xDataScaled[1], yDataScaled[1]],
            [xDataScaled[2], yDataScaled[2]],
        ]

        return xData, yData, xDataScaled, yDataScaled, polyData, polyDataScaled

    def __plotModeler(self, modeler: SectionModeler):
        trianglesCoords = modeler.getTrianglesCoords()

        # We need this for operate in data space
        refLenght = math.sqrt(modeler.calcSolidAreaProperties().area)

        ########################################################################
        #                               TRIANGLES                              #
        ########################################################################
        for k in trianglesCoords:
            coords = trianglesCoords[k]
            (
                xData,
                yData,
                xDataScaled,
                yDataScaled,
                polyData,
                polyDataScaled,
            ) = self.__genTriaData(coords)

            ####################################################################
            #                               FILL                               #
            ####################################################################
            if self.__plotTriaFill:
                self.__ax.add_patch(
                    Polygon(polyData, facecolor=self.__colorTriaFill, alpha=0.3)
                )

            ####################################################################
            #                               LINES                              #
            ####################################################################
            if self.__plotTriaLines:
                self.__ax.plot(xData, yData, color=self.__colorTriaLines)

            ####################################################################
            #                       SHRINK LINES AND FILL                      #
            ####################################################################
            if self.__plotTriaShrinkFill:
                self.__ax.add_patch(
                    Polygon(polyDataScaled, facecolor=self.__colorTriaFill, alpha=0.3)
                )

            if self.__plotTriaShrink:
                self.__plotScaled(xDataScaled, yDataScaled, refLenght)

        ########################################################################
        #                               SHAPE                                  #
        ########################################################################
        trianglesIds = modeler.getTrianglesIds()
        if self.__plotShapeEdges:
            segsId = []
            for k in trianglesIds:
                t = trianglesIds[k]
                # Order by id segments
                seg1 = [t[1], t[0]]
                if t[1] > t[0]:
                    seg1 = [t[0], t[1]]
                if seg1 not in segsId:
                    segsId.append(seg1)
                else:
                    segsId.remove(seg1)

                seg2 = [t[2], t[1]]
                if t[2] > t[1]:
                    seg2 = [t[1], t[2]]
                if seg2 not in segsId:
                    segsId.append(seg2)
                else:
                    segsId.remove(seg2)

                seg3 = [t[0], t[2]]
                if t[0] > t[2]:
                    seg3 = [t[2], t[0]]
                if seg3 not in segsId:
                    segsId.append(seg3)
                else:
                    segsId.remove(seg3)

            for ids in segsId:
                self.__ax.plot(
                    [modeler.getNodeX(ids[0]), modeler.getNodeX(ids[1])],
                    [modeler.getNodeY(ids[0]), modeler.getNodeY(ids[1])],
                    color=self.__colorEdge,
                )

        ########################################################################
        #                                NODES                                 #
        ########################################################################
        if self.__plotNodes or self.__plotSolidNodes:
            if self.__plotNodes:
                nodes = modeler.getNodes()
            else:
                nodes = modeler.getSolidNodes()

            for k in nodes:
                node = nodes[k]
                sc = self.__ax.scatter(
                    node.xn,
                    node.yn,
                    marker="o",
                    color=self.__colorNodes,
                    # linewidths = 3.5
                )
                sizeOfAnnotation = math.sqrt(sc.get_sizes()) / 2 + 2

                text_to_put = ""
                if self.__plotStresses and self.__plotSolidNodesId:
                    stress = self.__vertexStresses[node.id]
                    text_to_put = f"{stress:.1f}"
                else:
                    text_to_put = f"{node.id}"

                if self.__plotNodesId or self.__plotSolidNodesId:
                    self.__ax.annotate(
                        text=text_to_put,
                        xy=(node.xn, node.yn),
                        fontsize=self.__textSize,
                        xytext=(sizeOfAnnotation, sizeOfAnnotation),
                        textcoords="offset points",
                        color="red",
                        ha="left",
                        va="center",
                    )

        ########################################################################
        #                                REBAR                                 #
        ########################################################################
        texts = []
        patches = []
        if self.__plotRebar:
            circles = modeler.getCircles()
            for k in circles:
                circle = circles[k]
                x = circle.center.xn
                y = circle.center.yn
                r = circle.radius
                patch = self.__ax.add_patch(Circle((x, y), r))
                self.__ax.add_patch(Circle((x, y), r, fill=False, edgecolor="black"))
                patches.append(patch)

                if self.__plotStresses and self.__plotRebarId:
                    stress = self.__rebarStresses[circle.id]
                    text_to_put = f"{stress:.0f}"
                else:
                    text_to_put = f"{circle.id}"

                if self.__plotRebarId:
                    # text = self.__ax.annotate(
                    #     text=f"{text_to_put}",
                    #     xy=(x + r, y + r),
                    #     fontsize=self.__textSize,
                    #     # xytext = (sizeOfAnnotation, sizeOfAnnotation),
                    #     # textcoords="offset points",
                    #     ha="left",
                    #     va="center",
                    # )
                    text = self.__ax.text(
                        s=f"{text_to_put}",
                        x=x, # + r,
                        y=y, # + r,
                        fontsize=self.__textSize,
                        # xytext = (sizeOfAnnotation, sizeOfAnnotation),
                        # textcoords="offset points",
                        ha="left",
                        va="center",
                    )
                    texts.append(text)
        #self.__fig.canvas.draw()
        adjust_text(
            texts=texts,
            objects=patches,
            ax=self.__ax,
            max_move=100,
            expand=(1.2, 1.2),
            arrowprops=dict(arrowstyle='->', color='red', linewidth=0.3),
            force_static=(2.0,2.0))

        ########################################################################
        #                               MESHES                                 #
        ########################################################################
        if self.__plotMeshTria:
            for i in range(modeler.meshSize()):
                trianglesCoords = modeler.getTrianglesCoordsAtMesh(i)
                for k in trianglesCoords:
                    coords = trianglesCoords[k]
                    (
                        xData_,
                        yData_,
                        xDataScaled_,
                        yDataScaled_,
                        polyData_,
                        polyDataScaled_,
                    ) = self.__genTriaData(coords)
                    self.__plotScaled(xDataScaled_, yDataScaled_, refLenght)
                    self.__ax.plot(xData_, yData_, color=self.__colorTriaLines)

    def setView(self, opt: SectionPlotView_Enum):
        if opt == SectionPlotView_Enum.SP_VIEW_MESH:
            self.__plotShapeEdges: bool = False
            self.__plotTriaLines: bool = False
            self.__plotTriaFill: bool = False
            self.__plotTriaShrinkFill: bool = False
            self.__plotTriaShrink: bool = False
            self.__plotNodes: bool = False
            self.__plotNodesId: bool = False
            self.__plotRebar: bool = False
            self.__plotMeshTria: bool = True
            self.__plotRebarId: bool = False
            self.__plotGrid: bool = False

        if opt == SectionPlotView_Enum.SP_VIEW_SECTION:
            self.__plotShapeEdges = True
            self.__plotTriaLines = False
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = False
            self.__plotTriaShrink = False
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebar = True
            self.__plotRebarId = True
            self.__plotStresses = False

        if opt == SectionPlotView_Enum.SP_VIEW_SECTION_NOID:
            self.__plotShapeEdges = True
            self.__plotTriaLines = False
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = False
            self.__plotTriaShrink = False
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebar = True
            self.__plotRebarId = False

        if opt == SectionPlotView_Enum.SP_HIDE_REBAR_ID:
            self.__plotRebarId = False

        if opt == SectionPlotView_Enum.SP_VIEW_NODES:
            self.__plotNodes = True
            self.__plotNodesId = True

        if opt == SectionPlotView_Enum.SP_HIDE_NODES:
            self.__plotNodes = False
            self.__plotNodesId = False

        if opt == SectionPlotView_Enum.SP_VIEW_SOLID_NODES:
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotSolidNodes = True
            self.__plotSolidNodesId = True
            self.__plotStresses = False

        if opt == SectionPlotView_Enum.SP_VIEW_STRESSES:
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotRebarId = True
            self.__plotSolidNodes = True
            self.__plotSolidNodesId = True
            self.__plotStresses = True

        if opt == SectionPlotView_Enum.SP_HIDE_NODES:
            self.__plotNodes = False
            self.__plotNodesId = False
            self.__plotSolidNodes = False
            self.__plotSolidNodesId = False

        if opt == SectionPlotView_Enum.SP_VIEW_GRID:
            self.__plotGrid = True

        if opt == SectionPlotView_Enum.SP_HIDE_GRID:
            self.__plotGrid = False

        if opt == SectionPlotView_Enum.SP_VIEW_TRIANGLES:
            self.__plotShapeEdges = False
            self.__plotTriaLines = True
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = True
            self.__plotTriaShrink = True
            self.__plotNodes = True
            self.__plotNodesId = True
            self.__plotRebar = False
            self.__plotRebarId = False

        if opt == SectionPlotView_Enum.SP_VIEW_TRIANGLE_NOID:
            self.__plotShapeEdges = False
            self.__plotTriaLines = True
            self.__plotTriaFill = True
            self.__plotTriaShrinkFill = True
            self.__plotTriaShrink = True
            self.__plotNodes = True
            self.__plotNodesId = False
            self.__plotRebar = False
            self.__plotRebarId = False

    def plot(self):
        self.__ax.cla()
        #self.__ax.set_facecolor("white")
        if self.__plotGrid:
            self.__ax.grid(True, "both")
            self.__ax.minorticks_on()
            self.__ax.set_facecolor("#EBEBEB")

        if self.__modeler is not None:
            self.__plotModeler(self.__modeler)
        else:
            log("ERR", "Modeler does not assigned !!!", self.__ll)

    @staticmethod
    def show():
        plt.show()

    @staticmethod
    def save(
        fname,
        *,
        transparent=None,
        dpi="figure",
        format=None,
        metadata=None,
        bbox_inches=None,
        pad_inches=0.1,
        facecolor=None,
        edgecolor=None,
        backend=None,
        **kwargs,
    ):
        plt.savefig(
            fname,
            dpi=dpi,
            format=format,
            metadata=metadata,
            bbox_inches=bbox_inches,
            pad_inches=pad_inches,
            facecolor=facecolor,
            edgecolor=edgecolor,
            backend=backend,
            transparent=transparent,
            **kwargs,
        )


class Geometry2DPlot:
    def __init__(
        self,
        nrows: int = 1,
        ncols: int = 1,
        figsize: Tuple[float, float] = (7, 7),
        sx: float = 1,
        sy: float = 1,
    ):
        self.__fig = plt.figure(figsize=figsize)

        # Adjust layout
        # self.__fig.subplots_adjust(hspace=0.9, wspace=0.9)

        self.__ax: List[Axes] = [
            self.__fig.add_subplot(nrows, ncols, i + 1) for i in range(nrows * ncols)
        ]
        self.__titles: List[str] = ["" for i in range(len(self.__ax))]
        self.__xlabels: List[str] = ["" for i in range(len(self.__ax))]
        self.__ylabels: List[str] = ["" for i in range(len(self.__ax))]

        # Data to plot
        self.__points: List[Point2d] = []
        self.__pointArray: List[List[Point2d]] = []
        self.__pointClouds: List[PointCloud2d] = []

        # Scale Factor
        self.__sx = sx
        self.__sy = sy

        # Aspect controls
        self.__pointSize: float = 30
        self.__textSize: float = 12
        self.__showPointIndex: bool = True
        self.__showPointIndexArray: bool = True
        self.__showLineArray: bool = False
        self.__showArrow: bool = True
        self.__showLine: bool = True
        self.__colorLines: str = "black"
        self.__colorPoints: str = "red"
        self.__colorPointsArray: List[List[str]] = []

    @property
    def pointSize(self):
        return self.__pointSize

    @pointSize.setter
    def pointSize(self, value: float):
        self.__pointSize = value

    @property
    def textSize(self):
        return self.__textSize

    @textSize.setter
    def textSize(self, value: float):
        self.__textSize = value

    def setXLabel(self, xlabel: List[str]):
        self.__xlabels = xlabel

    def setYLabel(self, ylabel: List[str]):
        self.__ylabels = ylabel

    def setTitles(self, titles: List[str]):
        self.__titles = titles

    def showItems(
        self,
        pointIndex: bool = True,
        arrow: bool = True,
        line: bool = True,
        lineArray: bool = False,
        pointIndexArray: bool = True,
    ):
        self.__showPointIndex = pointIndex
        self.__showArrow = arrow
        self.__showLine = line
        self.__showLineArray = lineArray
        self.__showPointIndexArray = pointIndexArray

    def addPoint(self, point: Point2d) -> None:
        self.__points.append(point)

    def addPointArray(self, array: List[Point2d], colorArray: List[str] = []) -> None:
        self.__pointArray.append(array)

        if len(colorArray) == 0:
            self.__colorPointsArray.append(["black" for i in range(len(array))])
            return

        if len(colorArray) == 1:
            self.__colorPointsArray.append([colorArray[0] for i in range(len(array))])
            return

        if len(colorArray) not in [0, 1, len(array)]:
            raise ValueError(
                f"Len of array {len(array)} must be equal to \
                            len of colorArray {len(colorArray)}"
            )
        else:
            self.__colorPointsArray.append(colorArray)
            return

    def addPointCloud(self, cloud: PointCloud2d) -> None:
        self.__pointClouds.append(cloud)

    def __plotFromPointArray(self, points: List[List[Point2d]]):
        if len(self.__ax) != len(points):
            print("ERR: Can't plot many set of points with one axes")

        else:
            for idx, pts in enumerate(points):
                # save options
                showLineOld = self.__showLine
                self.__showLine = self.__showLineArray

                showPointIndexOld = self.__showPointIndex
                self.__showPointIndex = self.__showPointIndexArray

                self.__plotFromPoint2dLst(
                    pts, self.__ax[idx], self.__colorPointsArray[idx]
                )

                # restore options
                self.__showLine = showLineOld
                self.__showPointIndex = showPointIndexOld

    def __plotFromPoint2dLst(
        self, points: List[Point2d], axes: Axes, color: Union[str, List[str]]
    ):
        for i, p in enumerate(points):
            if isinstance(color, str):
                sc = axes.scatter(
                    [p.x * self.__sx],
                    [p.y * self.__sy],
                    color=color,
                    s=self.__pointSize,
                )
            else:
                sc = axes.scatter(
                    [p.x * self.__sx],
                    [p.y * self.__sy],
                    color=color[i],
                    s=self.__pointSize,
                )

            if self.__showPointIndex:
                sizeOfAnnotation = math.sqrt(sc.get_sizes()[0]) / 2 + 2
                axes.annotate(
                    text=f"{i}",
                    xy=(p.x * self.__sx, p.y * self.__sy),
                    xytext=(sizeOfAnnotation, 0),
                    fontsize=self.__textSize,
                    textcoords="offset points",
                    ha="left",
                    va="center",
                )

            if i < len(points) - 1:
                p0 = points[i]
                p1 = points[i + 1]

                if self.__showLine:
                    axes.plot(
                        [p0.x * self.__sx, p1.x * self.__sx],
                        [p0.y * self.__sy, p1.y * self.__sy],
                        color=self.__colorLines,
                    )

                if self.__showArrow:
                    centerOfArrow = [
                        (p0.x * self.__sx + p1.x * self.__sx) / 2,
                        (p0.y * self.__sy + p1.y * self.__sy) / 2,
                    ]
                    direction = [
                        (p1.x * self.__sx - p0.x * self.__sx),
                        (p1.y * self.__sx - p0.y),
                    ]
                    lenght = math.sqrt(
                        math.pow(direction[0], 2) + math.pow(direction[1], 2)
                    )
                    if not lenght == 0:
                        versor = [
                            direction[0] / lenght / 100,
                            direction[1] / lenght / 100,
                        ]
                        axes.annotate(
                            "",
                            xy=(centerOfArrow[0], centerOfArrow[1]),
                            xytext=(
                                centerOfArrow[0] + versor[0],
                                centerOfArrow[1] + versor[1],
                            ),
                            fontsize=self.__textSize * 2,
                            arrowprops={
                                "arrowstyle": "<-",
                            },
                        )
                    else:
                        print(
                            f"Point at coords {p0} duplicate with index {i} and {i + 1}"
                        )

    def __plotFromPointCloudLst(self, points: List[PointCloud2d]):

        if len(self.__titles) == len(self.__ax):
            for i, t in enumerate(self.__titles):
                self.__ax[i].set_title(t)
        else:
            if len(self.__titles) == 1:
                for a in self.__ax:
                    a.set_title(self.__titles[0])
            else:
                print("ERR: Can't set titles because lenght !!!")

        if len(self.__xlabels) == len(self.__ax):
            for i, t in enumerate(self.__xlabels):
                self.__ax[i].set_xlabel(t)
        else:
            if len(self.__xlabels) == 1:
                for a in self.__ax:
                    a.set_xlabel(self.__xlabels[0])
            else:
                print("ERR: Can't set xlabels because lenght !!!")

        if len(self.__ylabels) == len(self.__ax):
            for i, t in enumerate(self.__ylabels):
                self.__ax[i].set_ylabel(t)
        else:
            if len(self.__ylabels) == 1:
                for a in self.__ax:
                    a.set_ylabel(self.__ylabels[0])
            else:
                print("ERR: Can't set ylabels because lenght !!!")

        if len(self.__ax) == 1:
            for p in points:
                self.__plotFromPoint2dLst(
                    p.getPoints(), self.__ax[0], self.__colorPoints
                )

        else:
            if len(self.__ax) != len(points):
                print("ERR: Can't plot many set of points with one axes")

            else:
                for idx, p in enumerate(points):
                    self.__plotFromPoint2dLst(
                        p.getPoints(), self.__ax[idx], self.__colorPoints
                    )

    def plot(self):

        # Points
        self.__plotFromPoint2dLst(self.__points, self.__ax[0], self.__colorPoints)

        # Point cloud
        self.__plotFromPointCloudLst(self.__pointClouds)

        # Point array
        if len(self.__pointArray) > 0:
            self.__plotFromPointArray(self.__pointArray)

        self.__fig.tight_layout(pad=2)

    @staticmethod
    def show():
        plt.show()

    @staticmethod
    def save(
        fname,
        *,
        transparent=None,
        dpi="figure",
        format=None,
        metadata=None,
        bbox_inches=None,
        pad_inches=0.1,
        facecolor="auto",
        edgecolor="auto",
        backend=None,
        **kwargs,
    ):

        plt.savefig(
            fname,
            dpi=dpi,
            format=format,
            metadata=metadata,
            bbox_inches=bbox_inches,
            pad_inches=pad_inches,
            facecolor=facecolor,
            edgecolor=edgecolor,
            backend=backend,
            transparent=transparent,
            **kwargs,
        )


# def interactionDomainBasePlot2d(vertices, fields, xLabel = 'Nz [N]', yLabel = 'Mz [Nmm]', titleAddStr = None,
# tensionPoints = None, scale_Nx = 0.001, scale_Mz = 0.000001):
def interactionDomainBasePlot2d(vertices, fields, **kwargs):
    if "export" in kwargs:
        fileName = kwargs["export"]
    else:
        fileName = None

    if "dpi" in kwargs:
        dpi = kwargs["dpi"]
    else:
        dpi = None

    if "xLabel" in kwargs:
        xLabel = kwargs["xLabel"]
    else:
        xLabel = "Nz [N]"

    if "yLabel" in kwargs:
        yLabel = kwargs["yLabel"]
    else:
        yLabel = "Mz [Nmm]"

    if "titleAddStr" in kwargs:
        titleAddStr = kwargs["titleAddStr"]
    else:
        titleAddStr = None

    if "tensionPoints" in kwargs:
        tensionPoints = kwargs["tensionPoints"]
    else:
        tensionPoints = None

    if "scale_Nx" in kwargs:
        scale_Nx = kwargs["scale_Nx"]
    else:
        scale_Nx = 0.001

    if "scale_Mz" in kwargs:
        scale_Mz = kwargs["scale_Mz"]
    else:
        scale_Mz = 0.000001

    if "lines" in kwargs:
        lines = kwargs["lines"]
    else:
        lines = []

    if "markers" in kwargs:
        markers = kwargs["markers"]
    else:
        markers = True

    if "savingSingleDomains" in kwargs:
        savingSingleDomains = kwargs["savingSingleDomains"]
    else:
        savingSingleDomains = False

    if "hotPoints" in kwargs:
        hotPoints = kwargs["hotPoints"]
    else:
        hotPoints = False

    if "printDomains" in kwargs:
        printDomains = kwargs["printDomains"]
        if printDomains is None:
            printDomains = list(range(0, len(vertices)))
    else:
        printDomains = list(range(0, len(vertices)))

    """
    if "pointsScale" in kwargs:
        pointsScale = kwargs["pointsScale"]
    else:
        pointsScale = 40
    """
    if isinstance(vertices, list) and len(vertices) != 0:
        # Only one diagrammpython
        if isinstance(vertices[0][0], float) and isinstance(vertices[0][1], float):
            verticesList = [vertices]
            fieldsList = [fields]
        else:
            verticesList = vertices
            fieldsList = fields
    else:
        raise ValueError(
            "Vertices must be a list with lenght > 0 !!! Maybe you need build domain."
        )

    fig, ax = plt.subplots()

    if tensionPoints is not None:
        if len(tensionPoints) > 0:
            tp = np.array(tensionPoints, float)
            ax.scatter(
                tp[:, 0] * scale_Nx,
                tp[:, 1] * scale_Mz,
                marker="+",
                s=2000,
                color=[0, 0, 0],
            )

    # plt.autoscale(enable = None)
    plt.xlabel(xLabel)
    plt.ylabel(yLabel)
    plt.xticks(rotation="vertical")
    plt.margins(0.0)
    plt.subplots_adjust(bottom=0.25)
    plt.subplots_adjust(left=0.20)

    if len(lines) != 0:
        for ll in lines:
            x1 = ll[0].x * scale_Nx
            y1 = ll[0].y * scale_Mz
            x2 = ll[1].x * scale_Nx
            y2 = ll[1].y * scale_Mz
            plt.plot([x1, x2], [y1, y2], "k-", lw=2)

    strTitle = "Interaction Domain Nx - Mz"
    if titleAddStr is not None:
        plt.subplots_adjust(top=0.87)
        strTitle = strTitle + "\n" + titleAddStr

    ax.set_title(strTitle)
    ax.grid(True)
    ax.yaxis.set_major_formatter(FormatStrFormatter("%.2e"))
    ax.xaxis.set_major_formatter(FormatStrFormatter("%.2e"))

    # Cloning list only for plot only with deep
    verticesListCopy = copy.deepcopy(verticesList)

    # Removing columns from 2 to n that contains others values
    for _idx, vertices_i in enumerate(verticesListCopy):
        for idy, v in enumerate(vertices_i):
            vertices_i[idy] = [v[0], v[1]]

    for idx, vertices_i in enumerate(verticesListCopy):

        if idx not in printDomains:
            continue

        codes = [Path.MOVETO] + [Path.LINETO] * (len(vertices_i) - 1)

        vertices_i = np.array(vertices_i, float)

        # setting scale
        vertices_i[:, 0] = vertices_i[:, 0] * scale_Nx
        vertices_i[:, 1] = vertices_i[:, 1] * scale_Mz

        # setting color for hot and cold section
        _edgecolor = "black"
        if isinstance(hotPoints, bool):
            if hotPoints is False:
                _edgecolor = "black"
        if isinstance(hotPoints, list):
            if hotPoints[idx] is False:
                _edgecolor = "black"
            else:
                _edgecolor = "red"

        path = Path(vertices_i, codes)
        pathpatch = PathPatch(path, facecolor="None", edgecolor=_edgecolor)

        ax.add_patch(pathpatch)

        if markers:
            x, y = zip(*path.vertices)  # ?

            colorsMap = np.array(
                [
                    [1, 0, 0, 0.5],
                    [0, 1, 0, 0.5],
                    [0, 0, 1, 0.5],
                    [1, 0, 1, 0.5],
                    [1, 0, 0, 1.0],
                    [0, 1, 0, 1.0],
                    [0, 0, 1, 1.0],
                    [1, 0, 1, 1.0],
                ]
            )

            # Displayng Domain Different colors
            scatt_colors = []
            for f in fieldsList[idx]:
                if f == 1.0:
                    scatt_colors.append(0)
                elif f == 2.0:
                    scatt_colors.append(1)
                elif f == 3.0:
                    scatt_colors.append(2)
                elif f == 4.0:
                    scatt_colors.append(3)
                elif f == 11.0:
                    scatt_colors.append(4)
                elif f == 12.0:
                    scatt_colors.append(5)
                elif f == 13.0:
                    scatt_colors.append(6)
                elif f == 14.0:
                    scatt_colors.append(7)
                else:
                    raise ValueError("The value of f is <%1.4f> not mapped!!!" % f)

            # print('Lunghezza x = %f - Lunghezza scatt_colors %f '%(len(x),len(scatt_colors)))
            ax.scatter(x, y, marker="o", s=40, c=colorsMap[scatt_colors])

        if savingSingleDomains:
            ax.autoscale_view()
            if titleAddStr is not None:
                plt.savefig(titleAddStr + "_domain_" + str("%1.0i" % idx) + ".png")

    ax.autoscale_view()

    if fileName is not None:
        if dpi is not None:
            print("save figure to file name -->", fileName)
            plt.style.use("classic")
            plt.savefig(fileName, bbox_inches="tight", dpi=600)
        else:
            raise ValueError("If fileName is not None must provide dpi argument")
    else:
        plt.show()



import random
import unittest
from pathlib import Path

from pycivil.EXAGeometry.clouds import PointCloud2d
from pycivil.EXAGeometry.geometry import Point2d
from pycivil.EXAStructural.plot import Geometry2DPlot
from pycivil.EXAStructural.templateRCRect import RCTemplRectEC2


class Test(unittest.TestCase):
    def test_001_addPoint(self):
        gp = Geometry2DPlot()

        nb = 12
        ray = 1

        for _i in range(nb):
            gp.addPoint(
                Point2d(
                    random.uniform(-ray / 2, ray / 2), random.uniform(-ray / 2, ray / 2)
                )
            )

        gp.plot()
        gp.save(Path(__file__).parent / "test_001_addPoint.png")

    def test_002_addPointCloud(self):
        gp = Geometry2DPlot()

        nb = 12
        ray = 1

        points = []
        for _i in range(nb):
            points.append(
                Point2d(
                    random.uniform(-ray / 2, ray / 2), random.uniform(-ray / 2, ray / 2)
                )
            )
        pc = PointCloud2d(points)
        assert pc.isConvex() is False
        gp.addPointCloud(pc)

        points = []
        for _i in range(nb):
            points.append(
                Point2d(
                    random.uniform(-ray / 2 + 2, ray / 2 + 2),
                    random.uniform(-ray / 2 + 2, ray / 2 + 2),
                )
            )
        pc = PointCloud2d(points)
        gp.addPointCloud(pc)

        gp.plot()
        gp.save(Path(__file__).parent / "test_002_addPointCloud.png")

    def test_003_savePointCloud(self):

        points = []
        points.append(Point2d(-0.33155, -0.23683))
        points.append(Point2d(-0.34277, -0.35870))
        points.append(Point2d(0.31207, -0.21037))
        points.append(Point2d(-0.06377, -0.38116))
        points.append(Point2d(0.02957, 0.06710))
        points.append(Point2d(-0.25647, 0.36642))
        points.append(Point2d(0.33481, -0.13742))
        points.append(Point2d(-0.01376, -0.32877))
        points.append(Point2d(0.05573, 0.24965))
        points.append(Point2d(0.42329, -0.33455))
        points.append(Point2d(-0.14461, -0.39083))
        points.append(Point2d(-0.06779, -0.09793))

        pc = PointCloud2d(points)

        pc.save(Path(__file__).parent / "test_003_savePointCloud.pkl")

    def test_004_openPointCloud(self):
        points = []
        points.append(Point2d(-0.33155, -0.23683))
        points.append(Point2d(-0.34277, -0.35870))
        points.append(Point2d(0.31207, -0.21037))
        points.append(Point2d(-0.06377, -0.38116))
        points.append(Point2d(0.02957, 0.06710))
        points.append(Point2d(-0.25647, 0.36642))
        points.append(Point2d(0.33481, -0.13742))
        points.append(Point2d(-0.01376, -0.32877))
        points.append(Point2d(0.05573, 0.24965))
        points.append(Point2d(0.42329, -0.33455))
        points.append(Point2d(-0.14461, -0.39083))
        points.append(Point2d(-0.06779, -0.09793))

        pc0 = PointCloud2d(points)
        pc1 = PointCloud2d()
        pc1.open(Path(__file__).parent / "test_003_savePointCloud.pkl")

        self.assertEqual(pc0, pc1)

    def test_005_domain_01(self):
        section = RCTemplRectEC2(1, "First Section")
        section.setDimH(500.0)
        section.setDimW(300.0)
        section.addSteelArea("MB", 40.0, 600.0)
        section.addSteelArea("MT", 40.0, 600.0)
        section.setMaterials("C32/40", "B450C")
        pc = section.interactionDomainBuild2d(nbPoints=24)
        assert isinstance(pc, PointCloud2d)

        pc.save(Path(__file__).parent / "test_005_domain_01.pkl")
        pcFromFile = PointCloud2d()
        pcFromFile.open(Path(__file__).parent / "test_005_domain_01.pkl")

        self.assertEqual(pc, pcFromFile)
        self.assertTrue(isinstance(pc, PointCloud2d))

        print(f"Convexity: --> {pc.isConvex()}")
        gp = Geometry2DPlot()
        gp.addPointCloud(pc)
        gp.pointSize = 10
        gp.textSize = 10
        gp.showItems(True, False, True)
        gp.plot()
        gp.save(Path(__file__).parent / "test_005_domain_01.png")


if __name__ == "__main__":
    unittest.main()



"""
Created on Fri Jan 11 16:37:47 2019

@author: lpaone
"""
import copy
import unittest

from pycivil.EXAGeometry.geometry import (
    Edge2d,
    Edge3d,
    Node2d,
    Node3d,
    Point2d,
    Point3d,
    Polyline2d,
    Polyline3d,
    Seg2d,
    Vector2d,
    Vector3d,
    affineSum2d,
    areaFromTria3D,
    twoPointsDivide,
    twoPointsOffset,
)
from pycivil.EXAGeometry.shapes import Frame, ShapeCircle, ShapePoly


class Test(unittest.TestCase):
    def test_00_Points2d_Coordinates(self):

        # try:
        #     import EXADisplay as vis
        # except:
        #     print("EXADisplay error !!!")

        p1 = Point2d(0, 0)
        p2 = Point2d(1, 2.0 / 3)
        p3 = Point2d(1, 2)

        self.assertEqual(p1.x, 0)
        self.assertEqual(p1.y, 0)

        self.assertEqual(p2.x, 1)
        self.assertEqual(p2.y, 2.0 / 3)

        self.assertEqual(p1.x, 0)
        self.assertEqual(p1.y, 0)

        self.assertEqual(p3.x, 1)
        self.assertEqual(p3.y, 2)

        p1 = Point2d(coords=(0, 0))
        p2 = Point2d(coords=(1, 2.0 / 3))
        p3 = Point2d(coords=(1, 2))

        self.assertEqual(p1.x, 0)
        self.assertEqual(p1.y, 0)

        self.assertEqual(p2.x, 1)
        self.assertEqual(p2.y, 2.0 / 3)

        self.assertEqual(p1.x, 0)
        self.assertEqual(p1.y, 0)

        self.assertEqual(p3.x, 1)
        self.assertEqual(p3.y, 2)

        # Polyline3D testing start
        nodesLst = [
            Node3d(1, 0.0, 0.0, 0),
            Node3d(2, 300.0, 0.0, 0),
            Node3d(3, 300.0, 500.0, 0),
            Node3d(4, 0.0, 500.0, 0),
        ]
        poly3 = Polyline3d(nodesLst)

        poly3.setClosed()
        self.assertTrue(poly3.isClosed())

        # Op for Point3d
        p1 = Point3d(0.0, 0.0, 1.0)
        p2 = Point3d(1.0, 1.0, 0.0)
        p3 = p1 + p2

        # print("p1+p2 is: %s\n" % p3)

        # Op for Node3d
        n1 = Node3d(0.0, 0.0, 1.0)
        n2 = Node3d(1.0, 1.0, 0.0)
        # print(n1)
        # print(n2)
        # print("n1+n2 is: %s\n" % n3)

        # Op for Vector3d
        v1 = Vector3d(0.0, 0.0, 1.0)
        v2 = Vector3d(1.0, 1.0, 0.0)
        # print(v1)
        # print(v2)
        # print("v1+v2 is: %s\n" % v3)

        # Cross vector for Vector3d
        v1 = Vector3d(1.0, 0.0, 0.0)
        v2 = Vector3d(0.0, 1.0, 0.0)
        # print(v1)
        # print(v2)
        v1.cross(v2)
        # print("v1.cross(v2) is: %s\n" % v3)

        # Norm of vector for Vector3d
        v1 = Vector3d(5.0, 0.0, 0.0)
        # print(v1)
        # print("v1.norm() is: %d\n" % v1.norm())

        # Normalizition of vector for Vector3d
        v1.normalize()
        # print("v1.normalize() is: %s" % v1)

        # Vector 3d from two points
        p1 = Point3d(1.0, 0.0, 0.0)
        p2 = Point3d(2.0, 0.0, 0.0)
        # print(p1)
        # print(p2)
        v1 = Vector3d(p1, p2)
        # print("v1 from two points: %s\n" % v1)

        # Generic Shape Abstraact
        # sec = Shape()
        # print(sec)

        # Frame with Shape
        nodesLst = [
            Node2d(0.0, 0.0, 1),
            Node2d(300.0, 0.0, 2),
            Node2d(300.0, 500.0, 3),
            Node2d(0.0, 500.0, 4),
        ]
        poly = Polyline2d(nodesLst)
        shape = ShapePoly(poly)

        edge1 = Edge3d(Node3d(1, 1, 1, 1), Node3d(2, 3, 3, 3))
        frame1 = Frame(edge1)

        frame1.setShape(shape)
        # print(frame1)

        # Build a normal frame
        # print("We want build a typical frame 3D")

        n1 = Node3d(1.0, 1.0, 1.0, 1)
        n2 = Node3d(6.0, 6.0, 6.0, 2)

        axis = Edge3d(n1, n2)

        frame_typical = Frame(axis)

        frame_typical.setReference(Point3d(1.0, 1.0, 0.0))
        frame_typical.setReference(1.0, 1.0, 0.0)
        # print(frame_typical)

        # print(frame_typical.getXLocalAxis())
        # print(frame_typical.getYLocalAxis())
        # print(frame_typical.getZLocalAxis())

        # Polygonar
        secPoly = ShapePoly(0.0, 0.0, 1, 300.0, 0.0, 2, 300.0, 500.0, 3, 0.0, 500.0, 4)

        frame_typical.setShape(secPoly)

        # print(frame_typical)

        secPoly.translate(Point2d(150.0, 250.0), Point2d(0.0, 0.0))

        # print(frame_typical)

        axis = Edge3d(1.0, 1.0, 1.0, 1, 6.0, 6.0, 6.0, 2)

        # print(axis)

        frame_typical = Frame(100.0, 100.0, 100.0, 1, 600.0, 600.0, 600.0, 2)
        frame_typical.setReference(100.0, 100.0, 0.0)
        frame_typical.setShape(secPoly)

        frame_typical2 = Frame(15.0, 25.0, 35.0, 1, 60.0, 70.0, 90.0, 2)
        frame_typical2.setReference(15.0, 25.0, 10.0)
        frame_typical2.setShape(secPoly)

        # print(frame_typical)

        # Op for Vector2d
        v1 = Vector2d(vx=0.0, vy=0.0)
        v2 = Vector2d(vx=1.0, vy=1.0)
        # print(v1)
        # print(v2)
        # print("v1+v2 is: %s\n" % v3)

        # Cross vector for Vector2d
        v1 = Vector2d(vx=1.0, vy=0.0)
        v2 = Vector2d(vx=0.0, vy=1.0)
        # print(v1)
        # print(v2)
        v1.cross(v2)
        # print("v1.cross(v2) is: %s\n" % v3)

        # Norm of vector for Vector2d
        v1 = Vector2d(vx=5.0, vy=0.0)
        # print(v1)
        # print("v1.norm() is: %d\n" % v1.norm())

        # Normalizition of vector for Vector2d
        v1.normalize()
        # print("v1.normalize() is: %s" % v1)

        # Vector 2d from two points
        p1 = Point2d(1.0, 0.0)
        p2 = Point2d(2.0, 0.0)
        # print(p1)
        # print(p2)
        v1 = Vector2d(p1, p2)
        # print("v1 from two points: %s\n" % v1)

        # Seg2d from two points
        Seg2d(p1, p2)
        # print("Distance betwen p1 and p2 is %1.6f" % p1.distance(p2))
        # print(seg)

        self.assertRaises(ValueError, Point2d, None, None)
        # if pnull.isNull():
        #     print("Point Null Point2d(None,None) is Null !!!")

        # if not p1.isNull():
        #     print("Point p1 Point2d(1.0,0.0) is not Null !!!")

        # visualizer = vis.Visualizer()
        # visualizer.addStruComponent(frame_typical)
        # visualizer.addStruComponent(secPoly)
        # visualizer.display()

        # vis.test_extrusion()
        # vis.test_sec()
        # vis.test_vector_sec()
        # vis.test_axis()
        # vis.test_transform()

    def test_01_Points2d_scalarVector(self):
        p1 = Point2d(1, 2)
        p2 = 2 * p1
        self.assertEqual(p2.x, 2, "x coordinate")
        self.assertEqual(p2.y, 4, "y coordinate")

        p3 = p2 * 2
        self.assertEqual(p3.x, 4, "x coordinate")
        self.assertEqual(p3.y, 8, "y coordinate")

    def test_02_Node2d_properties(self):

        # instance
        node = Node2d(101, 2.3, 4)

        self.assertEqual(node.x, 101)
        self.assertEqual(node.y, 2.3)
        self.assertEqual(node.idn, 4)

        # modified
        node.x = 1
        node.y = 2
        node.idn = 1

        self.assertEqual(node.x, 1)
        self.assertEqual(node.y, 2)
        self.assertEqual(node.idn, 1)

        # default args
        node = Node2d()
        self.assertEqual(node.x, 0.0)
        self.assertEqual(node.y, 0.0)
        self.assertEqual(node.idn, -1)

    def test_03_Polyline2d_properties(self):
        n1 = Node2d(0.0, 0.0, 1)
        n2 = Node2d(300.0, 0.0, 2)
        n3 = Node2d(300.0, 500.0, 3)
        n4 = Node2d(0.0, 500.0, 4)

        # Bad formed
        with self.assertRaises(TypeError):
            Polyline2d([n1, n2], 2, 3)

        # Well formed
        Polyline2d([n1, n2, n3, n4])

    def test_04_Polyline2d_methods(self):

        # Polyline2D testing start
        nodesLst = [
            Node2d(0.0, 0.0, 1),
            Node2d(300.0, 0.0, 2),
            Node2d(300.0, 500.0, 3),
            Node2d(0.0, 500.0, 4),
        ]
        poly2 = Polyline2d(nodesLst)

        self.assertEqual(poly2.size(), 4)
        self.assertFalse(poly2.isClosed())

        poly2.setClosed()
        self.assertEqual(poly2.size(), 5)
        self.assertTrue(poly2.isClosed())

    def test_05_Edge2d_properties(self):
        n1 = Node2d(0.0, 0.0, 1)
        n2 = Node2d(300.0, 0.0, 2)

        with self.assertRaises(TypeError):
            Edge2d(n1)

        with self.assertRaises(TypeError):
            Edge2d(n1, 1)

        edge1 = Edge2d(n1, n2)

        ni = edge1.nodeI()
        nj = edge1.nodeJ()

        self.assertEqual(ni.x, 0.0)
        self.assertEqual(ni.y, 0.0)
        self.assertEqual(ni.idn, 1)

        self.assertEqual(nj.x, 300.0)
        self.assertEqual(nj.y, 0.0)
        self.assertEqual(nj.idn, 2)

    def test_06_Node3d_properties(self):

        # instance
        node = Node3d(1, 2, 3, 4)

        self.assertEqual(node.x, 1)
        self.assertEqual(node.y, 2)
        self.assertEqual(node.z, 3)
        self.assertEqual(node.idn, 4)

        # defalut args
        node = Node3d()

        self.assertEqual(node.x, 0.0)
        self.assertEqual(node.y, 0.0)
        self.assertEqual(node.z, 0.0)
        self.assertEqual(node.idn, -1)

    def test_07_Edge3d_properties(self):
        edge1 = Edge3d(Node3d(1, 1, 1, 1), Node3d(2, 3, 3, 3))
        edge2 = Edge3d(Node3d(1, 3, 3, 3), Node3d(2, 3, 3, 0))

        self.assertEqual(edge1.nodeI().x, 1)
        self.assertEqual(edge1.nodeI().y, 1)
        self.assertEqual(edge1.nodeI().z, 1)
        self.assertEqual(edge1.nodeI().idn, 1)

        self.assertEqual(edge1.nodeJ().x, 2)
        self.assertEqual(edge1.nodeJ().y, 3)
        self.assertEqual(edge1.nodeJ().z, 3)
        self.assertEqual(edge1.nodeJ().idn, 3)

        self.assertEqual(edge2.nodeI().x, 1)
        self.assertEqual(edge2.nodeI().y, 3)
        self.assertEqual(edge2.nodeI().z, 3)
        self.assertEqual(edge2.nodeI().idn, 3)

        self.assertEqual(edge2.nodeJ().x, 2)
        self.assertEqual(edge2.nodeJ().y, 3)
        self.assertEqual(edge2.nodeJ().z, 3)
        self.assertEqual(edge2.nodeJ().idn, 0)

    def test_08_Vector2d_properties(self):

        # instance 2 float args
        v = Vector2d(vx=1.0, vy=2.0)
        self.assertEqual(v.vx, 1.0)
        self.assertEqual(v.vy, 2.0)

        # instance 2 int args
        v = Vector2d(vx=1, vy=2)
        self.assertEqual(v.vx, 1)
        self.assertEqual(v.vy, 2)

        # instance 2 int and float args
        v = Vector2d(vx=1, vy=2.0)
        self.assertEqual(v.vx, 1)
        self.assertEqual(v.vy, 2)

        # instance 2 args
        p1 = Point2d(1, 1)
        p2 = Point2d(2, 3)
        v = Vector2d(p1, p2)
        self.assertEqual(v.vx, 1)
        self.assertEqual(v.vy, 2)

        # default args
        v = Vector2d()
        self.assertEqual(v.vx, 0.0)
        self.assertEqual(v.vy, 0.0)

        with self.assertRaises(TypeError, msg="not sended"):
            Vector2d(1, 2, 3)

        with self.assertRaises(TypeError, msg="not sended"):
            Vector2d(1)

        with self.assertRaises(TypeError, msg="not sended"):
            Vector2d(1, Point2d())

    def test_09_Vector3d_properties(self):

        # instance 3 float args
        v = Vector3d(1.0, 2.0, 3.0)
        self.assertEqual(v.vx, 1.0)
        self.assertEqual(v.vy, 2.0)
        self.assertEqual(v.vz, 3.0)

        # instance 3 int args
        v = Vector3d(1, 2, 3)
        self.assertEqual(v.vx, 1)
        self.assertEqual(v.vy, 2)
        self.assertEqual(v.vz, 3)

        # instance 2 args
        p1 = Point3d(1, 1, 1)
        p2 = Point3d(2, 3, 4)
        v = Vector3d(p1, p2)
        self.assertEqual(v.vx, 1)
        self.assertEqual(v.vy, 2)
        self.assertEqual(v.vz, 3)

        # default args
        v = Vector3d()
        self.assertEqual(v.vx, 0.0)
        self.assertEqual(v.vy, 0.0)
        self.assertEqual(v.vz, 0.0)

    def test_10_Frame_properties(self):
        edge1 = Edge3d(Node3d(1, 1, 1, 1), Node3d(2, 3, 3, 3))
        edge2 = Edge3d(Node3d(1, 3, 3, 3), Node3d(2, 3, 3, 0))

        Frame(edge1)
        Frame(edge2)

        with self.assertRaises(TypeError, msg="not sended"):
            Frame(1)

        with self.assertRaises(TypeError, msg="not sended"):
            Frame(1, 1)

    def test_11_ShapePoly_properties(self):

        nodesLst = [
            Node2d(0.0, 0.0, 1),
            Node2d(300.0, 0.0, 2),
            Node2d(300.0, 500.0, 3),
            Node2d(0.0, 500.0, 4),
        ]
        poly = Polyline2d(nodesLst)

        ShapePoly(poly)

    def test_12_MidPoint_2D(self):
        v0 = Point2d(0.0, 0.0)
        v1 = Point2d(1.0, 0.0)
        self.assertEqual(v0.midpoint(v1).x, 0.5)
        self.assertEqual(v0.midpoint(v1).y, 0.0)

    def test_13_AreaFromTria_2D(self):
        v0 = Point2d(0.0, 0.0)
        v1 = Point2d(1.0, 0.0)
        v2 = Point2d(0.5, 1.0)
        self.assertEqual(
            Point2d.areaFromTria(v0, v1, v2),
            v0.distance(v1) * v0.midpoint(v1).distance(v2) / 2,
        )

    def test_14_AreaFromTria_3D(self):
        v0 = Point2d(0.0, 0.0)
        v1 = Point2d(1.0, 0.0)
        v2 = Point2d(0.5, 1.0)
        area2D = Point2d.areaFromTria(v0, v1, v2)

        p0 = Point3d(0.0, 0.0, 2.0)
        p1 = Point3d(1.0, 0.0, 2.0)
        p2 = Point3d(0.5, 1.0, 2.0)
        area3D_1 = areaFromTria3D(p0, p1, p2)

        self.assertEqual(area2D, area3D_1)

        p0 = Point3d(0.0, 2.0, 0.0)
        p1 = Point3d(1.0, 2.0, 0.0)
        p2 = Point3d(0.5, 2.0, 1.0)
        area3D_2 = areaFromTria3D(p0, p1, p2)

        self.assertEqual(area2D, area3D_2)

    def test_15_shape_circle(self):

        # First constructor
        shape1 = ShapeCircle(20, 100, 200)
        self.assertEqual(shape1.getRadius(), 20)
        self.assertEqual(shape1.getShapePoint("O").x, 100)
        self.assertEqual(shape1.getShapePoint("O").y, 200)

        # Second constructor
        shape2 = ShapeCircle(20, center=Point2d(101, 201))
        self.assertEqual(shape2.getRadius(), 20)
        self.assertEqual(shape2.getShapePoint("O").x, 101)
        self.assertEqual(shape2.getShapePoint("O").y, 201)

    def test_16_vector_rotate(self):
        v1 = Vector2d(Point2d(0, 0), Point2d(5, 5))
        v2 = copy.deepcopy(v1).rotate(90)
        self.assertAlmostEqual(v2.vx, -5)
        self.assertAlmostEqual(v2.vy, +5)

        self.assertAlmostEqual(v1.normalize().cross(v2.normalize()), 1)

    def test_17_sum_affine(self):
        p = Point2d(1, 1)
        v = Vector2d(vx=2, vy=3)
        psum = affineSum2d(p, v)
        self.assertEqual(psum.x, 3)
        self.assertEqual(psum.y, 4)

    def test_18_points_divide(self):
        p0 = Point2d(1, 1)
        p1 = Point2d(2, 2)
        self.assertRaises(ValueError, twoPointsDivide, p0, p1, 0)

        arr1 = twoPointsDivide(p0, p1, 1)
        self.assertTrue(len(arr1) == 2)
        self.assertTrue(arr1[0] == p0)
        self.assertTrue(arr1[1] == p1)

        arr2 = twoPointsDivide(p0, p1, 4)
        self.assertTrue(len(arr2) == 5)
        self.assertTrue(arr2[0] == p0)
        self.assertTrue(arr2[1] == Point2d(1.25, 1.25))
        self.assertTrue(arr2[2] == Point2d(1.50, 1.50))
        self.assertTrue(arr2[3] == Point2d(1.75, 1.75))
        self.assertTrue(arr2[4] == p1)

    def test_19_points_offset(self):
        p0 = Point2d(1, 1)
        p1 = Point2d(1, 2)
        pp0, pp1 = twoPointsOffset(p0, p1, 10)
        self.assertAlmostEqual(pp0.x, Point2d(-9, 1).x)
        self.assertAlmostEqual(pp0.y, Point2d(-9, 1).y)
        self.assertAlmostEqual(pp1.x, Point2d(-9, 2).x)
        self.assertAlmostEqual(pp1.y, Point2d(-9, 2).y)


if __name__ == "__main__":
    unittest.main()



"""
Created on Sun Aug 18 09:57:40 2019

@author: lpaone
"""

from pycivil.EXAStructural import templateRCRect as est

# import EXAStructural as es

# Setting new instance of section with
# id = 1 and name = "First Section"
section = est.RCTemplRectEC2(1, "First Section")
section.setElementType("beam")

# Setting dimension concrete
section.setDimH(500.0)
section.setDimW(300.0)

# Setting rebar
section.addSteelArea("MT", 40.0, 500.0)
section.addSteelArea("MB", 40.0, 1000.0)

# Setting materials
concrete = "C32/40"
steel = "B450C"
sigmacMax = 0.6 * 32
sigmasMax = 0.8 * 450
homogeneization = 15.0
section.setMaterials(concrete, steel, sigmacMax, sigmasMax, homogeneization)

print("Area di calcestruzzo: %1.3E [mm^2]" % section.calConcreteArea())
print("Area di acciaio:      %1.3E [mm^2]\n" % section.calSteelArea())

section.getSection()

print("Baricentro acciaio:")
print("------------------")
print(section.calBarycenterOfSteel())
print("\nBaricentro calcestruzzo:")
print("-----------------------")
print(section.calBarycenterOfConcrete())
print("\nBaricentro sezione omogeneizzata:")
print("--------------------------------")
print(section.calBarycenter())
print("\nArea calcestruzzo   : %1.3E" % section.calProp_Ac())
print("Area acciaio        : %1.3E" % section.calProp_As())
print("Area omogeneizzata  : %1.3E" % section.calProp_Ah())
print("Proprietà Scx       : %1.3E" % section.calProp_Scx())
print("Proprietà Ssx       : %1.3E" % section.calProp_Ssx())
print("Proprietà Shx       : %1.3E" % section.calProp_Shx())
print("Proprietà Scy       : %1.3E" % section.calProp_Scy())
print("Proprietà Ssy       : %1.3E" % section.calProp_Ssy())
print("Proprietà Shy       : %1.3E" % section.calProp_Shy())
print("Proprietà Icx       : %1.3E" % section.calProp_Icx())
print("Proprietà Isx       : %1.3E" % section.calProp_Isx())
print("Proprietà Ihx       : %1.3E" % section.calProp_Ihx())
print("Proprietà Icy       : %1.3E" % section.calProp_Icy())
print("Proprietà Isy       : %1.3E" % section.calProp_Isy())
print("Proprietà Ihy       : %1.3E" % section.calProp_Ihy())


sol = section.solverSLS_NM(N=-500.0 * 1e3, M=100.0 * 1e6)
print(f"sigma_c = {sol[0]:1.3E} - sigma_s = {sol[1]:2.3E} - xi = {sol[2]:3.3E} ")
print("\nCalcestruzzo con N < 0:")
print("-----------------------")
print(section.getConcrStress())
print("\nAcciaio con N < 0:")
print("-----------------")
print(section.getSteelStress())

sol = section.solverSLS_NM(N=+500.0 * 1e3, M=200.0 * 1e6)
print(sol)
print(f"sigma_c = {sol[0]:1.3E} - sigma_s = {sol[1]:2.3E} - xi = {sol[2]:3.3E} ")
print("\nCalcestruzzo con N > 0:")
print("-----------------------")
print(section.getConcrStress())
print("\nAcciaio con N > 0:")
print("-----------------")
print(section.getSteelStress())



"""
Created on Sat Dec 31 13:10:00 2022

@author: lpaone
"""
import json
import math
import os
import shutil
import tempfile
import pytest
import sys
from pathlib import Path
from uuid import uuid4

from code_aster_whale.tools.code_aster import AsterLauncher as ca
from code_aster_whale.tools.code_aster import ExportType, LauncherType

from pycivil.EXAGeometry.geometry import Point3d
from pycivil.settings import ServerSettings
from pycivil.EXAStructural import templateRCRect as est
from pycivil.EXAStructural.templateRCRectFire import (
    RCRectEC2FireDesign as SectionForFireDesign,
)
from pycivil.EXAStructural.templateRCRectFire import RMinutes as FireTime
from pycivil.EXAUtils.vtk import FileType
from pycivil.EXAUtils.vtk import Parser as vtkParser

@pytest.mark.codeaster
def test_00_launcher():
    appName = "test_00_launcher"
    jobToken = f"test-{str(uuid4())}"
    # Build launcher
    #
    launcher = ca(
        launcherType=LauncherType.CONTAINER,
        exportType=ExportType.VTK_FROM_MEDCOUPLING,
        appName=appName,
        jobToken=jobToken,
        containerName=ServerSettings().codeaster_container,
    )
    # Run Code Aster
    # NOTE: use test = True to post file
    #
    pathName = os.path.dirname(__file__)
    source_EXPORT_file = os.path.join(pathName, "thermNL01.export")
    source_COMM_file = os.path.join(pathName, "thermNL01.comm")
    source_MMED_file = os.path.join(pathName, "thermNL01.mmed")
    assert 0 == launcher.launch(
            exportFilePath=source_EXPORT_file,
            commFilePath=source_COMM_file,
            mmedFilePath=source_MMED_file,
            test=True), f"Can't launch Code Aster on {source_EXPORT_file}"
    # Exporting with option timeStep = -1 means that will be exported
    # complete time series
    #
    assert 0 == launcher.exportFromMEDFile(
            os.path.join(pathName, "thermNL01.rmed"),
            pointDataArr=["resther0TEMP"],
            test=True)
    # TODO: Adjust with matplotlib. Export images with pvbatch from thermal analysis
    #
    # launcher.exportImgsFromVtm(pathName, "vtm", pathName)
    logLevel = 3
    parser = vtkParser()
    parser.setFileType(FileType.VTU_TYPE)
    parser.parse(files=os.path.join(pathName, "vtu"), ll=logLevel)
    # k = parser.findMultiblockKeyByArrayValue(
    #     arrayName="TimeValue", value=6000.0, ll=logLevel
    # )
    # parser.setCurrentMultiblockByKey(key=k)
    parser.setCurrentFileByIndex(int(6000.0 / 100 / 60))
    parser.getPoints()
    parser.getPoint(300)
    parser.getPointValueById(arrayName="resther0TEMP", id=2, ll=logLevel)
    # Retrive value in point by nearest point
    print("---------------------------------------:")
    print("Retrive value in point by nearest point:")
    print("---------------------------------------:")
    testPoint = Point3d(0.15, -0.30, 0)
    pointFound = parser.getNearestPoint(p=testPoint)
    value = parser.getPointValueById(
        arrayName="resther0TEMP", id=pointFound.idn, ll=logLevel
    )
    print(
        "Distance beetwen point is {:.5f} m".format(
            pointFound.distanceFrom(testPoint)
        )
    )
    print(f"---> Temp in point id = {pointFound.idn} is {value:.1f}")
    print("---------------------------------------:")
    # Retrive value in point by interp on cellPoints
    print("-----------------------------------------------:")
    print("Retrive value in point by interp on cell points:")
    print("-----------------------------------------------:")
    testPoint = Point3d(+0.15, -0.30, 0)
    pointsFound = parser.getPointsCellNearestPoint(p=testPoint)
    value = parser.getPointValueInterpTriaByIds(
        p=testPoint,
        id0=pointsFound[0].idn,
        id1=pointsFound[1].idn,
        id2=pointsFound[2].idn,
        arrayName="resther0TEMP",
        ll=logLevel,
    )
    print(f"---> Temp in point is {value:.1f}")
    print("---------------------------------------:")
    # launcher.deleteArtifacts()

@pytest.mark.codeaster
def test_01_not_regression_domain():
    section = est.RCTemplRectEC2(1, "Template RC Section")
    section.setMaterials(
        concreteStr="C30/37", steelStr="B450C", homogenization=15.0
    )
    section.setDimH(600.0)
    section.setDimW(300.0)
    d20 = math.pi * 20 * 20 / 4
    d16 = math.pi * 16 * 16 / 4
    section.addSteelArea("XY", area=d20, x=-150.0 + 20, y=-300.0 + 20)
    section.addSteelArea("XY", area=d20, x=-150.0 + 100, y=-300.0 + 20)
    section.addSteelArea("XY", area=d20, x=+150.0 - 100, y=-300.0 + 20)
    section.addSteelArea("XY", area=d20, x=+150.0 - 20, y=-300.0 + 20)
    section.addSteelArea("XY", area=d16, x=-150.0 + 20, y=+300.0 - 20)
    section.addSteelArea("XY", area=d16, x=-150.0 + 100, y=+300.0 - 20)
    section.addSteelArea("XY", area=d16, x=+150.0 - 100, y=+300.0 - 20)
    section.addSteelArea("XY", area=d16, x=+150.0 - 20, y=+300.0 - 20)
    # Adding Tension Points
    KN = 1000
    KNm = 1000000
    section.addTensionPoint2d(N=500.0 * KN, M=191.2 * KNm)
    section.addTensionPoint2d(N=0.0 * KN, M=104.0 * KNm)
    section.interactionDomainBuild2d(nbPoints=100)
    sectionFd = SectionForFireDesign(
        section,
        logLevel=3,
        jobToken=f"test-{str(uuid4())}",
        codeAsterLauncher=ServerSettings().codeaster_launcher,
        codeAsterTemplatePath=ServerSettings().codeaster_templates_path,
        codeAsterContainerName=ServerSettings().codeaster_container,
    )
    sectionFd.setLogLevel(1)
    # Opening JSON file
    f = open(os.path.join(os.path.dirname(__file__), "notregression.json"))
    # returns JSON object as a dictionary
    data = json.load(f)
    f.close()
    #
    # build temporary dir for test
    #
    tmpdirname = tempfile.mkdtemp(suffix=None, prefix=None, dir=None)
    print(f"Temporary path --> {tmpdirname}")
    sectionFd.setWorkingPath(tmpdirname, True)
    assert sectionFd.buildMesh(), "Building mesh failed"
    sectionFd.setEsposedBottom(True)
    sectionFd.setEsposedLeft(False)
    sectionFd.setEsposedRight(False)
    sectionFd.setEsposedTop(False)
    assert sectionFd.buildThermalMap(test=True), "Building thermal maps"
    # TODO: need to fix it
    #
    # sectionFd.exportThermalImgs()
    sectionFd.setTime(FireTime.R180)
    sectionFd.parse()
    sectionFd.buildHotSection()
    sectionFd.deleteArtifacts()
    assert sectionFd.getSection() is not None
    section.interactionDomainBuild2d(nbPoints=200, hotPoints=True)
    # ------------------------------------------------
    # using this for generate not regression json file
    # ------------------------------------------------
    # thisPath = os.path.dirname(os.path.abspath(__file__))
    # with open(os.path.join(thisPath, "notregression.json"), "w") as outfile:
    #     json.dump(section.getInteractionDomainDict(), outfile, indent=4)
    assert dict(data) == section.getInteractionDomainDict()
    fireFactorsNotRegression = [
        0.05688497578862946,
        0.05689855014757066,
        0.05677341540764942,
        0.05679166542990918,
        1.0,
        1.0,
        1.0,
        1.0,
    ]
    steelTempsNotRegression = [
        915.5751210568527,
        915.5072492621467,
        916.1329229617529,
        916.0416728504541,
        20.000342941411276,
        20.0004018915307,
        20.000457508601922,
        20.00052850487601,
    ]
    for idx in range(len(steelTempsNotRegression)):
        assert fireFactorsNotRegression[idx] == pytest.approx(section.getSteelFireFactors()[idx], abs=2)
        assert steelTempsNotRegression[idx] == pytest.approx(section.getSteelTemperatures()[idx], abs=2)
        assert sectionFd.getDeltas()[0] == pytest.approx( 0.000, abs=2)
        assert sectionFd.getDeltas()[1] == pytest.approx( 0.000, abs=2)
        assert sectionFd.getDeltas()[2] == pytest.approx( 0.000, abs=2)
        assert sectionFd.getDeltas()[3] == pytest.approx(61.314, abs=2)
    # -----------------------------------
    # using this for generate plot window
    # -----------------------------------
    # self.__section.interactionDomainPlot2d(xLabel = 'N [KN]', yLabel = 'M [KN*m]')
    #
    # remove temporary dir for test
    #
    shutil.rmtree(tmpdirname)


if __name__ == "__main__":
    sys.exit(pytest.main(Path(__file__)))



import sys
import pytest
from pathlib import Path

from pycivil.EXAGeotechnical import base as geot
from pycivil.EXAParametric import tube as param

def test_001(tmp_path: Path) -> None:

    E_top = geot.ModulusYoung()
    E_top.setChoose(9)
    Et = (E_top.value()[2] + E_top.value()[2]) / 2 * 0.1

    E_bot = geot.ModulusYoung()
    E_bot.setChoose(2)
    Eb = (E_bot.value()[2] + E_bot.value()[2]) / 2 * 0.1

    ni_top = geot.RatioPoisson()
    ni_top.setChoose(5)
    nit = (ni_top.value()[2] + ni_top.value()[2]) / 2

    ni_bot = geot.RatioPoisson()
    ni_bot.setChoose(3)
    nib = (ni_bot.value()[2] + ni_bot.value()[2]) / 2

    soilTop = geot.SoilLayer("Terreno superiore - Ghiaia")
    soilTop.setEt(Et)
    soilTop.setNit(nit)
    soilTop.setDry(gamma=20, phi=30, coe=0)
    print(soilTop)

    soilBot = geot.SoilLayer("Terreno inferiore - Argilla Limosa")
    soilBot.setEt(Eb)
    soilBot.setNit(nib)
    soilBot.setCu(0.05)
    print(soilBot)

    title = "CENTROPADANA - tombino sottostrada - manufatto I.62 - DN 1000"
    circle = param.CircularTubeShape01(
        descr=title, B=1000 + 200 + 300, H=1000 + 100 + 300 + 200, Tw=200, Tt=200, Tb=200
    )

    circle.assignTopLayer(soilTop)
    circle.assignBotLayer(soilBot)

    circle.springs()
    print(circle)

    circle.assignSeismicAction(0.261, 0.2)
    circle.setTopCover(7600.0 - 600.0 + 150)
    circle.setPaverThickness(600.0, 20.0)

    circle.buildFEModel()

    fileName = "I02"
    fileExtension = ".mgt"
    circle.exportFEModel(str(tmp_path / f"{fileName}"), fileExtension)
    filePath = tmp_path / f"{fileName}{fileExtension}"
    circle.model().clear()

    assert filePath.exists()

if __name__ == "__main__":
    sys.exit(pytest.main(Path(__file__)))



import json
import os
import unittest

from pycivil.EXAStructural.checkable import Checker
from pycivil.EXAStructural.codes import Code
from pycivil.EXAStructural.loads import ForcesOnSection as Load
from pycivil.EXAStructural.loads import (
    Frequency_Enum,
    LimiteState_Enum,
)
from pycivil.EXAStructural.materials import Concrete, ConcreteSteel
from pycivil.EXAStructural.templateRCRect import RCTemplRectEC2
from pycivil.EXAStructuralCheckable.RcsRectangular import RcsRectangular


class Test(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def test_110_Rectangular_Shape_getArea(self):
        # Setting code for check
        code = Code("NTC2018")

        concrete = Concrete(descr="My concrete")
        concrete.setByCode(code, "C25/30")
        concrete.setEnvironmentNotAggressive()

        # Setting code for check
        steel = ConcreteSteel(descr="My steel")
        steel.setByCode(code, "B450C")
        steel.setEnvironmentNotSensitive()

        # Setting units
        KN = 1000
        KNm = 1000 * 1000

        # Loads on structural system
        force_1 = Load(Fx=+150.0 * KN, My=+150.0 * KNm, Fz=150 * KN, descr="force_1")
        force_1.frequency = Frequency_Enum.CHARACTERISTIC
        force_1.limitState = LimiteState_Enum.SERVICEABILITY

        force_2 = Load(Fx=+100.0 * KN, My=+145.0 * KNm, Fz=120 * KN, descr="force_2")
        force_2.frequency = Frequency_Enum.QUASI_PERMANENT
        force_1.limitState = LimiteState_Enum.SERVICEABILITY

        force_3 = Load(Fx=-100.0 * KN, My=+145.0 * KNm, Fz=120 * KN, descr="force_3")
        force_3.frequency = Frequency_Enum.QUASI_PERMANENT
        force_3.limitState = LimiteState_Enum.SERVICEABILITY

        force_4 = Load(Fx=+125.0 * KN, My=+125.0 * KNm, Fz=125 * KN, descr="force_4")
        force_4.frequency = Frequency_Enum.FREQUENT
        force_4.limitState = LimiteState_Enum.SERVICEABILITY

        force_5 = Load(Fx=-200.0 * KN, My=+200.0 * KNm, Fz=200 * KN, descr="force_5")
        force_5.limitState = LimiteState_Enum.ULTIMATE

        force_6 = Load(Fx=+200.0 * KN, My=-200.0 * KNm, Fz=100 * KN, descr="force_6")
        force_6.limitState = LimiteState_Enum.ULTIMATE

        force_7 = Load(Fx=-500.0 * KN, My=+350.0 * KNm, Fz=200 * KN, descr="force_7")
        force_7.limitState = LimiteState_Enum.ULTIMATE

        force_8 = Load(Fx=+100.0 * KN, My=-200.0 * KNm, Fz=70 * KN, descr="force_8")
        force_8.limitState = LimiteState_Enum.ULTIMATE

        force_9 = Load(Fx=+200.0 * KN, My=-219.0 * KNm, Fz=100 * KN, descr="force_9")
        force_9.limitState = LimiteState_Enum.ULTIMATE

        force_10 = Load(Fx=+1050.0 * KN, My=0.0 * KNm, Fz=100 * KN, descr="force_10")
        force_10.limitState = LimiteState_Enum.ULTIMATE

        force_11 = Load(Fx=-3700.0 * KN, My=0.0 * KNm, Fz=100 * KN, descr="force_10")
        force_11.limitState = LimiteState_Enum.ULTIMATE

        # Build checkable structural system
        section = RCTemplRectEC2(1, "Template RC Section")

        section.setConcreteMaterial(concrete)
        section.setSteelMaterial(steel)

        section.setDimH(600)
        section.setDimW(300)

        section.addSteelArea("LINE-MT", dist=20, d=20, nb=4, sd=40)
        section.addSteelArea("LINE-MB", dist=20, d=20, nb=4, sd=40)

        section.setStirrupt(area=100, dist=150, angle=90)

        # --------------------
        # BY-HAND TEST CASE #3
        # --------------------
        self.assertAlmostEqual(section.calConcreteArea(), 180000, 6)
        self.assertAlmostEqual(section.calSteelArea(), 2513.274123, 6)
        self.assertAlmostEqual(section.calBarycenterOfConcrete().y, 0, 6)
        self.assertAlmostEqual(section.calBarycenterOfSteel().y, 0, 6)
        # --------------------

        print(section.getConcreteMaterial())
        print(section.getSteelMaterial())

        checkable = RcsRectangular(section)

        locationArtifacts = os.path.dirname(__file__)
        fileNameAArtifact = os.path.basename(__file__).split(".")[0] + ".out.png"
        checkable.setOption_SLU_NM_save(
            True, filePath=locationArtifacts, fileName=fileNameAArtifact
        )

        # Use tool for check
        checker = Checker()
        checker.setCheckable(checkable)

        # Perform check
        criteria = ["SLE-NM", "SLE-F", "SLU-T", "SLU-NM"]
        forces = [
            force_1,
            force_2,
            force_3,
            force_4,
            force_5,
            force_6,
            force_7,
            force_8,
            force_9,
            force_10,
            force_11,
        ]
        loadsInCriteria = [[0, 1, 2, 3], [2, 3], [4], [4, 5, 6, 7, 8, 9, 10]]
        checker.check(criteria, forces, code, loadsInCriteria)

        # Retrive and print results
        results = checker.getResults()
        print(json.dumps(results, indent=2))

    # It does'nt do nothing !!!
    def test_210_Rectangular_single_check_SLU_NM(self):
        code = Code("NTC2018")

        concrete = Concrete(descr="My concrete")
        concrete.setByCode(code, "C25/30")
        concrete.setEnvironment("not aggressive")

        steel = ConcreteSteel(descr="My steel")
        steel.setByCode(code, "B450C")
        steel.setSensitivity("not sensitive")

        KN = 1000
        KNm = 1000 * 1000
        force = Load(Fx=-200.0 * KN, My=+200.0 * KNm, Fz=200 * KN, descr="single_force")
        force.limitState = LimiteState_Enum.ULTIMATE

        section = RCTemplRectEC2(1, "Template RC Section")
        section.setConcreteMaterial(concrete)
        section.setSteelMaterial(steel)

        section.setDimH(600)
        section.setDimW(300)

        section.addSteelArea("LINE-MT", dist=20, d=20, nb=4, sd=40)
        section.addSteelArea("LINE-MB", dist=20, d=20, nb=4, sd=40)
        checkable = RcsRectangular(section)
        checkable.check_SLU_NM_NTC2018(force)

    #
    # Produced 2025/07/18 launching massive computations with strand7 results
    #
    # WARNING: intersection with point P=(-115980.0-0.0) are 1 not 2 !!!
    # seg P1 = (1202520.830214809, -108043575.76061487) - P2 = (1433877.5234923575, -1.4901161193847656e-08)
    # Point2d: (1.43e+06, -1.49e-08)
    # WARNING: Safety factor is 0.0 for plate 2679 layer 3 combo_name SLU_16
    #        : Group Name: Model\Piedritti\ARM2_7_SX Thickness is 1160
    #        : Med = -0.0 Ned = -115.98 Rebar: [[5, 21.6, 200, -73], [5, 21.6, 200, 73]]

    def test_211_Rectangular_single_check_SLU_NM(self):
        code = Code("NTC2018")

        concrete = Concrete(descr="My concrete")
        concrete.setByCode(code, "C25/30")
        concrete.setEnvironment("not aggressive")

        steel = ConcreteSteel(descr="My steel")
        steel.setByCode(code, "B450C")
        steel.setSensitivity("not sensitive")

        KN = 1000
        KNm = 1000 * 1000

        force_1 = Load(Fx=-115.98 * KN, My=-0.0 * KNm, descr="single_force")
        force_1.limitState = LimiteState_Enum.ULTIMATE

        section = RCTemplRectEC2(1, "Template RC Section")

        section.setConcreteMaterial(concrete)
        section.setSteelMaterial(steel)

        section.setDimH(1080)
        section.setDimW(1000)

        section.addSteelArea("LINE-MT", dist=73, d=21.6, nb=5, sd=200)
        section.addSteelArea("LINE-MB", dist=73, d=21.6, nb=5, sd=200)
        checkable = RcsRectangular(section)
        checkable.setMaxPointsForSLUDomain(20)
        checkable.check_SLU_NM_NTC2018(force_1)

        locationArtifacts = os.path.dirname(__file__)
        fileNameAArtifact = os.path.basename(__file__).split(".")[0] + "_211.png"
        checkable.setOption_SLU_NM_save(
            True, filePath=locationArtifacts, fileName=fileNameAArtifact
        )
        # res = checkable.check_SLU_NM_NTC2018(force)
        # print(res)

        # Use tool for check
        checker = Checker()
        checker.setCheckable(checkable)

        # Perform check
        criteria = ["SLU-NM"]
        forces = [force_1]
        loadsInCriteria = [[0]]
        checker.check(criteria, forces, code, loadsInCriteria)

        # Retrive and print results
        results = checker.getResults()
        print(json.dumps(results, indent=2))
        assert (
                results["resultsForCriteria"][0]["results"][0]
                ["safetyFactor"]["interactionDomain"] > 0.0
        )

    #
    # Produced 2025/07/18 launching massive computations with strand7 results
    #
    # WARNING: intersection with point P=(348900.0, -97940000.0) are 3 not 2 !!!
    #          seg P1=(2619300.056842802, -129812832.21945465)-P2=(2974951.6519645965, 1.043081283569336e-07)
    # Point2d: (-8.14e+06,2.28e+09)
    # Point2d: (-8.14e+06,2.28e+09)
    # Point2d: (1.73e+06,-4.84e+08)
    # WARNING: Safety factor is 0.0 for plate 316 layer 4 combo_name SLU_11
    #        : Group Name: Model\Piedritti\ARM2_14_SX Thickness is 1000
    #        : Med = -97.94 Ned = 348.9 Rebar: [[10, 22, 200, -95], [10, 22, 200, 95]]

    def test_212_Rectangular_single_check_SLU_NM(self):
        code = Code("NTC2018")

        concrete = Concrete(descr="My concrete")
        concrete.setByCode(code, "C25/30")
        concrete.setEnvironment("not aggressive")

        steel = ConcreteSteel(descr="My steel")
        steel.setByCode(code, "B450C")
        steel.setSensitivity("not sensitive")

        KN = 1000
        KNm = 1000 * 1000

        force_1 = Load(Fx=348.9 * KN, My=-97.94 * KNm, descr="single_force")
        force_1.limitState = LimiteState_Enum.ULTIMATE

        section = RCTemplRectEC2(1, "Template RC Section")

        section.setConcreteMaterial(concrete)
        section.setSteelMaterial(steel)

        section.setDimH(920)
        section.setDimW(1000)

        section.addSteelArea("LINE-MT", dist=95, d=22, nb=10, sd=200)
        section.addSteelArea("LINE-MB", dist=95, d=22, nb=10, sd=200)
        checkable = RcsRectangular(section)
        checkable.setMaxPointsForSLUDomain(20)
        checkable.check_SLU_NM_NTC2018(force_1)

        locationArtifacts = os.path.dirname(__file__)
        fileNameAArtifact = os.path.basename(__file__).split(".")[0] + "_211.png"
        checkable.setOption_SLU_NM_save(
            True, filePath=locationArtifacts, fileName=fileNameAArtifact
        )

        # Use tool for check
        checker = Checker()
        checker.setCheckable(checkable)

        # Perform check
        criteria = ["SLU-NM"]
        forces = [force_1]
        loadsInCriteria = [[0]]
        checker.check(criteria, forces, code, loadsInCriteria)

        # Retrive and print results
        results = checker.getResults()
        print(json.dumps(results, indent=2))
        assert (
                results["resultsForCriteria"][0]["results"][0]
                ["safetyFactor"]["interactionDomain"] > 0.0
        )


    def test_213_Rectangular_single_check_SLU_NM(self):
        code = Code("NTC2018")

        concrete = Concrete(descr="My concrete")
        concrete.setByCode(code, "C25/30")
        concrete.setEnvironment("not aggressive")

        steel = ConcreteSteel(descr="My steel")
        steel.setByCode(code, "B450C")
        steel.setSensitivity("not sensitive")

        KN = 1000
        KNm = 1000 * 1000

        force_1 = Load(Fx=-712660.0, My=--102330000.0, descr="single_force")
        force_1.limitState = LimiteState_Enum.ULTIMATE

        section = RCTemplRectEC2(1, "Template RC Section")

        section.setConcreteMaterial(concrete)
        section.setSteelMaterial(steel)

        section.setDimH(1040)
        section.setDimW(1000)

        section.addSteelArea("LINE-MT", dist=95, d=16, nb=5, sd=200)
        section.addSteelArea("LINE-MB", dist=95, d=16, nb=5, sd=200)
        checkable = RcsRectangular(section)
        checkable.setMaxPointsForSLUDomain(20)
        checkable.check_SLU_NM_NTC2018(force_1)

        locationArtifacts = os.path.dirname(__file__)
        fileNameAArtifact = os.path.basename(__file__).split(".")[0] + "_212.png"
        checkable.setOption_SLU_NM_save(
            True, filePath=locationArtifacts, fileName=fileNameAArtifact
        )

        # Use tool for check
        checker = Checker()
        checker.setCheckable(checkable)

        # Perform check
        criteria = ["SLU-NM"]
        forces = [force_1]
        loadsInCriteria = [[0]]
        checker.check(criteria, forces, code, loadsInCriteria)

        # Retrive and print results
        results = checker.getResults()
        print(json.dumps(results, indent=2))
        assert (
                results["resultsForCriteria"][0]["results"][0]
                ["safetyFactor"]["interactionDomain"] > 0.0
        )


if __name__ == "__main__":
    unittest.main()



import math
from pathlib import Path

import pytest
from pycivil.EXAStructural.codes import Code
from pycivil.EXAStructural.materials import Concrete, ConcreteSteel
from pycivil.EXAStructural.templateRCRect import RCTemplRectEC2


# --------------------
# BY-HAND TEST CASE #3
# --------------------
def test_symmetrical_uncracked_NM_001(tmp_path: Path):
    # Setting code for check
    code = Code("NTC2018")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C25/30")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(600)
    rcSection.setDimW(300)

    # Longitudinal rebars
    lineMTRebars = rcSection.addSteelArea("LINE-MT", dist=50, d=20, nb=4, sd=40)
    lineMBRebars = rcSection.addSteelArea("LINE-MB", dist=50, d=20, nb=4, sd=40)

    # Geometrical properties
    assert pytest.approx(rcSection.calConcreteArea()) == 180000
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-3) == 2513
    assert pytest.approx(rcSection.calIdealArea(), rel=1e-3) == 217699
    assert pytest.approx(rcSection.calProp_Ihx(), rel=1e-3) == 7.755e09
    assert pytest.approx(
        [rcSection.calBarycenterOfConcrete().x, rcSection.calBarycenterOfConcrete().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenterOfSteel().x, rcSection.calBarycenterOfSteel().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenter().x, rcSection.calBarycenter().y]
    ) == [0.0, 0.0]

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [0, 0, None]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == 0
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == 0

    sigmac, sigmas, xi = rcSection.solverSLS_NM(1000 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [4.593, 68.895, math.inf]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == 4.593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == 68.895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(-1000 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-4.593, -68.895, math.inf]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == -4.593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == -68.895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(-100 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [
        -0.4593,
        -68.895 / 10,
        math.inf,
    ]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == -0.4593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == -6.8895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(+100 * KN, 0.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [+0.4593, +6.8895, math.inf]
    for i in rcSection.getConcrStress():
        assert pytest.approx(i, rel=1e-3) == +0.4593
    for i in rcSection.getSteelStress():
        assert pytest.approx(i, rel=1e-3) == +6.8895

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0.0 * KN, 150.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-5.803, 72.53, 300]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == +5.803
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == -5.803
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineMBRebars:
            stressExpected = +72.53
        if idx in lineMTRebars:
            stressExpected = -72.53
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-3) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0.0 * KN, -150.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-5.803, +72.53, 300]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == -5.803
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == +5.803
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineMBRebars:
            stressExpected = -72.53
        if idx in lineMTRebars:
            stressExpected = +72.53
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-3) == stressExpected

    # Neutral axis is measured from bottom to top of section
    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        -1000.0 * KN, +150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-10.396, +3.635, 537.45]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == +1.210
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -10.396
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineMBRebars:
            stressExpected = +3.635
        if idx in lineMTRebars:
            stressExpected = -141.425
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        -1000.0 * KN, -150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-10.396, +3.635, 62.65]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == -10.396
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == +1.210
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineMBRebars:
            stressExpected = -141.425
        if idx in lineMTRebars:
            stressExpected = +3.635
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        +1000.0 * KN, +150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-1.210, +141.425, 62.55]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == 10.396
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -1.210
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineMBRebars:
            stressExpected = +141.425
        if idx in lineMTRebars:
            stressExpected = -3.635
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected

    sigmac, sigmas, xi = rcSection.solverSLS_NM(
        +1000.0 * KN, -150.0 * KNm, uncracked=True
    )
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-1.210, +141.425, 537.45]
    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-2) == -1.210
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == 10.396
    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineMBRebars:
            stressExpected = -3.635
        if idx in lineMTRebars:
            stressExpected = +141.425
        print(
            f"Steel stress calculated: {i:.5f} equal to expected: {stressExpected:.5f}"
        )
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.2
# ----------------------
def test_uncracked_NM_002(tmp_path: Path):
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)

    # Geometrical properties
    assert pytest.approx(rcSection.calConcreteArea(), rel=1e-6) == 960.00e03
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-3) == 18.09e03
    assert pytest.approx(rcSection.calIdealArea(), rel=1e-3) == 1.23135e06
    assert pytest.approx(
        [rcSection.calBarycenterOfConcrete().x, rcSection.calBarycenterOfConcrete().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenterOfSteel().x, rcSection.calBarycenterOfSteel().y],
        rel=1e-2,
        abs=1e-3,
    ) == [0.0, -220.89]
    assert pytest.approx(
        [rcSection.calBarycenter().x, rcSection.calBarycenter().y], rel=1e-2, abs=1
    ) == [0.0, -49.39]
    assert (
        pytest.approx(rcSection.calProp_Ihx(barycenter=True), rel=1e-3, abs=1e-3)
        == 176.24e09
    )

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0 * KN, 2400.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-8.84, +96.97, 649.39]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == +7.4988
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == -8.840

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineRebars01:
            stressExpected = -117.11
        if idx in lineRebars02:
            stressExpected = +74.91
        if idx in lineRebars03:
            stressExpected = +85.94
        if idx in lineRebars04:
            stressExpected = +96.97
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.2
# ----------------------
def test_uncracked_NM_003(tmp_path: Path):
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=130, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=184, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)

    # Geometrical properties
    assert pytest.approx(rcSection.calConcreteArea(), rel=1e-6) == 960.00e03
    assert pytest.approx(rcSection.calSteelArea(), rel=1e-3) == 18.09e03
    assert pytest.approx(rcSection.calIdealArea(), rel=1e-3) == 1.23135e06
    assert pytest.approx(
        [rcSection.calBarycenterOfConcrete().x, rcSection.calBarycenterOfConcrete().y]
    ) == [0.0, 0.0]
    assert pytest.approx(
        [rcSection.calBarycenterOfSteel().x, rcSection.calBarycenterOfSteel().y],
        rel=1e-2,
        abs=1e-3,
    ) == [0.0, +220.89]
    assert pytest.approx(
        [rcSection.calBarycenter().x, rcSection.calBarycenter().y], rel=1e-2, abs=1
    ) == [0.0, +49.39]
    assert (
        pytest.approx(rcSection.calProp_Ihx(barycenter=True), rel=1e-3, abs=1e-3)
        == 176.24e09
    )

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0 * KN, -2400.0 * KNm, uncracked=True)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-8.84, +96.97, 550.61]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == -8.840
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == +7.4988

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineRebars04:
            stressExpected = -117.11
        if idx in lineRebars03:
            stressExpected = +74.91
        if idx in lineRebars02:
            stressExpected = +85.94
        if idx in lineRebars01:
            stressExpected = +96.97
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.5
# ----------------------
def test_cracked_stretched_NM_004(tmp_path: Path):
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(+3619 * KN, 0.0 * KNm, uncracked=False)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [0.0, +376.27, 1666.0]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == 0.0
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-3) == 0.0

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineRebars01:
            stressExpected = 376.27
        if idx in lineRebars02:
            stressExpected = 153.86
        if idx in lineRebars03:
            stressExpected = 141.05
        if idx in lineRebars04:
            stressExpected = 128.27
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.6
# ----------------------
def test_cracked_compressed_NM_005(tmp_path: Path):
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(-3619 * KN, 0.0 * KNm, uncracked=False)
    assert pytest.approx([sigmac, sigmas, xi], rel=1e-2) == [-3.599, -36.92, 3557.496]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == -2.385
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -3.599

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineRebars01:
            stressExpected = -52.84
        if idx in lineRebars02:
            stressExpected = -38.56
        if idx in lineRebars03:
            stressExpected = -37.74
        if idx in lineRebars04:
            stressExpected = -36.92
        assert pytest.approx(i, rel=1e-2) == stressExpected


# ----------------------
# BY-HAND TEST CASE #2.1
# ----------------------
def test_cracked_bending_NM_006(tmp_path: Path):
    # Setting code for check
    code = Code("NTC2008")

    concrete = Concrete(descr="My concrete")
    concrete.setByCode(code, "C32/40")

    # Setting code for check
    steel = ConcreteSteel(descr="My steel")
    steel.setByCode(code, "B450C")

    # Build checkable structural system
    rcSection = RCTemplRectEC2(1, "Template RC Section")

    # Concrete dimension
    rcSection.setDimH(1200)
    rcSection.setDimW(800)

    # Longitudinal rebars
    # Aggregates diameter maximum 25mm
    lineRebars01 = rcSection.addSteelArea("LINE-MT", dist=76, d=24, nb=10, sd=49)
    lineRebars02 = rcSection.addSteelArea("LINE-MT", dist=1016, d=24, nb=10, sd=49)
    lineRebars03 = rcSection.addSteelArea("LINE-MT", dist=1070, d=24, nb=10, sd=49)
    lineRebars04 = rcSection.addSteelArea("LINE-MT", dist=1124, d=24, nb=10, sd=49)

    # Setting units
    KN = 1000
    KNm = 1000 * 1000

    sigmac, sigmas, xi = rcSection.solverSLS_NM(0.0 * KN, 2400.0 * KNm, uncracked=False)

    assert pytest.approx([sigmac, sigmas, xi], rel=1e-3) == [-10.31, +207.10, 480.51]

    for idx, i in enumerate(rcSection.getConcrStress()):
        if idx < 2:
            # BL and BR
            assert pytest.approx(i, rel=1e-3) == 0.0
        else:
            # TL and TR
            assert pytest.approx(i, rel=1e-2) == -10.31

    for idx, i in enumerate(rcSection.getSteelStress()):
        stressExpected = 0
        if idx in lineRebars01:
            stressExpected = -130.19
        if idx in lineRebars02:
            stressExpected = +172.34
        if idx in lineRebars03:
            stressExpected = +189.72
        if idx in lineRebars04:
            stressExpected = +207.10
        assert pytest.approx(i, rel=1e-2) == stressExpected
